"""
$Id: module_posti.py 340 2012-11-14 22:16:45Z riku.lindblad@gmail.com $
$HeadURL: https://pyfibot.googlecode.com/svn/trunk/pyfibot/modules/module_posti.py $
"""

import urllib2
from BeautifulSoup import BeautifulSoup
import datetime
import time

baseurl = "http://www.verkkoposti.com/e3/TrackinternetServlet?lang=fi&LOTUS_hae=Hae&LOTUS_side=1&LOTUS_trackId=%s&LOTUS_hae=Hae"


def command_posti(bot, user, channel, args):
    """Get the latest status for a package"""
    result = getstatus(args, count=1)

    for line in result:
        return bot.say(channel, line.encode("UTF-8"))


def getstatus(code, count=None):
    url = baseurl % code
    f = urllib2.urlopen(url)
    d = f.read()
    f.close()

    bs = BeautifulSoup(d)

    res = []

    statuslist = bs.find("div", {'class': "result_up"}).find("table", {'width': '500'}).findAll("p", {'class': 'resulttext'})
    for status in statuslist:
        date, statustext, location = status.contents
        statustext = statustext.string
        date = time.strptime(date, "%d.%m.%Y, klo %H:%M&nbsp;")
        location = location[6:].strip()

        dt = datetime.datetime(*date[0:6])
        now = datetime.datetime.now()
        age = now - dt

        agestr = []

        if age.days > 0:
            agestr.append("%dd" % age.days)

        secs = age.seconds
        hours, minutes, seconds = secs // 3600, secs // 60 % 60, secs % 60

        if hours > 0:
            agestr.append("%dh" % hours)
        if minutes > 0:
            agestr.append("%dm" % minutes)

        res.append("%s - %s - %s" % (" ".join(agestr) + " ago", statustext, location))

    if count:
        return res[:count]
    else:
        return res



# -*- coding: utf-8 -*-

"""
$Id: module_forecast.py 331 2012-11-13 10:18:18Z riku.lindblad@gmail.com $
$HeadURL: https://pyfibot.googlecode.com/svn/trunk/pyfibot/modules/module_forecast.py $
"""

has_pywapi = False
try:
    import pywapi
    has_pywapi = True
except:
    print('Error loading library pywapi. Probably you havent installed it yet.')


def fahrenheit_to_celcius(f):
    return (int(f) - 32) / 1.8


def command_forecast(bot, user, channel, args):
    """This module tells weather forecast for location"""
    if not has_pywapi:
        return

    result_dict = pywapi.get_weather_from_google(args)
    if not all(result_dict.values()):
        bot.say(channel, 'unknown location')
        return

    def format_day(day):
        return (u'%s: %s (%.0f°C/%.0f°C)' % (day['day_of_week'], \
                          day['condition'], \
                          fahrenheit_to_celcius(day['low']), \
                          fahrenheit_to_celcius(day['high'])))

    answerstr = u'%s: ' % (result_dict['forecast_information']['city'])
    answerstr += u", ".join(format_day(day) for day in result_dict['forecasts'])

    bot.say(channel, answerstr.encode('utf-8'))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
A modular python bot based on the twisted matrix irc library

@author Riku 'Shrike' Lindblad (shrike@addiktit.net)
@copyright Copyright (c) 2006 Riku Lindblad
@license New-Style BSD

$Id: pyfibot.py 342 2012-11-14 23:20:23Z riku.lindblad@gmail.com $
$HeadURL: https://pyfibot.googlecode.com/svn/trunk/pyfibot/pyfibot.py $
"""

import sys
import os.path
import time
import urllib
import fnmatch
import HTMLParser
import logging
import logging.handlers
from gzip import GzipFile
from StringIO import StringIO

try:
    import yaml
except ImportError:
    print "PyYAML not found, please install from http://pyyaml.org/wiki/PyYAML"
    sys.exit(1)

# twisted imports
try:
    from twisted.internet import reactor, protocol, ssl
except ImportError:
    print "Twisted library not found, please install Twisted from http://twistedmatrix.com/products/download"
    sys.exit(1)

from util import timeoutdict
from BeautifulSoup import BeautifulSoup
from BeautifulSoup import UnicodeDammit

# default timeout for socket connections
import socket
socket.setdefaulttimeout(20)

import botcore

log = logging.getLogger('core')


class URLCacheItem(object):
    """URL cache item object, fetches data only when needed"""

    def __init__(self, url):
        self.url = url
        self.content = None
        self.headers = None
        self.bs = None
        # maximum size in kB to download
        self.max_size = 2048
        self.fp = None

    def _open(self, url):
        """Returns the raw file pointer to the given URL"""
        if not self.fp:
            urllib._urlopener = BotURLOpener()
            try:
                self.fp = urllib.urlopen(self.url)
            except IOError, e:
                log.warn("IOError (%s) when opening url %s", e, url)
        return self.fp

    def _checkstatus(self):
        """Check if all data has already been cached and close socket if so"""
        if self.content and self.headers and self.bs:
            self.fp.close()

    def getSize(self):
        """Get the content length of URL in kB

        @return None if the server doesn't return a content-length header"""
        if 'content-length' in self.getHeaders():
            length = int(self.getHeaders()['content-length']) / 1024
            return length
        else:
            return None

    def getContent(self):
        """Get the actual file at the URL

        @return None if the file is too large (over 2MB)"""
        if not self.content:
            f = self._open(self.url)

            size = self.getSize()
            if size > self.max_size:
                log.warn("CONTENT TOO LARGE, WILL NOT FETCH %s %s" % (size, self.url))
                self.content = None
            else:
                if self.checkType():
                    # handle gzipped content
                    if f.info().get('Content-Encoding') == 'gzip':
                        log.debug("Gzipped data, uncompressing")
                        buf = StringIO(f.read())
                        f = GzipFile(fileobj=buf)
                    self.content = UnicodeDammit(f.read()).unicode
                else:
                    type = self.getHeaders().getsubtype()
                    log.warn("WRONG CONTENT TYPE, WILL NOT FETCH size:%s, type:%s, %s" % (size, type, self.url))
        self._checkstatus()
        return self.content

    def getHeaders(self):
        """Get headers for the URL"""
        if not self.headers:
            f = self._open(self.url)
            if f:
                self.headers = f.info()
            else:
                self.headers = {}
        self._checkstatus()
        return self.headers

    def checkType(self):
        if self.getHeaders().getsubtype() in ['html', 'xml', 'xhtml+xml', 'atom+xml', 'json', 'javascript']:
            return True
        else:
            return False

    def getBS(self):
        """Get a beautifulsoup instance for the URL

        @return None if the url doesn't contain HTML
        """
        if not self.bs:
            # only attempt a bs parsing if the content is html, xml or xhtml
            if 'content-type' in self.getHeaders() and \
               self.getHeaders().getsubtype() in ['html', 'xml', 'xhtml+xml', 'atom+xml']:
                try:
                    bs = BeautifulSoup(markup=self.getContent())
                except HTMLParser.HTMLParseError:
                    log.warn("BS unable to parse content")
                    return None
                self.bs = bs
            else:
                return None
        self._checkstatus()
        return self.bs


class BotURLOpener(urllib.FancyURLopener):
    """URL opener that fakes itself as a regular browser and ignores all basic auth prompts"""
    def __init__(self, *args):
        # Latest Chrome on Windows XP
        self.version = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.16 Safari/534.30"
        urllib.FancyURLopener.__init__(self, *args)

    def prompt_user_passwd(self, host, realm):
        log.info("PASSWORD PROMPT:", host, realm)
        return ('', '')


class Network:
    def __init__(self, root, alias, address, nickname, channels=None, linerate=None, password=None, is_ssl=False):
        self.alias = alias                         # network name
        self.address = address                     # server address
        self.nickname = nickname                   # nick to use
        self.channels = channels or {}             # channels to join
        self.linerate = linerate
        self.password = password
        self.is_ssl = is_ssl

    def __repr__(self):
        return 'Network(%r, %r)' % (self.alias, self.address)


class InstantDisconnectProtocol(protocol.Protocol):
    def connectionMade(self):
        self.transport.loseConnection()


class ThrottledClientFactory(protocol.ClientFactory):
    """Client factory that inserts a slight delay to connecting and reconnecting"""
    lostDelay = 10
    failedDelay = 60

    def clientConnectionLost(self, connector, reason):
        log.info("connection lost (%s): reconnecting in %d seconds" % (reason, self.lostDelay))
        reactor.callLater(self.lostDelay, connector.connect)

    def clientConnectionFailed(self, connector, reason):
        log.info("connection failed (%s): reconnecting in %d seconds" % (reason, self.failedDelay))
        reactor.callLater(self.failedDelay, connector.connect)


class PyFiBotFactory(ThrottledClientFactory):
    """python.fi bot factory"""

    version = "20110425.0"
    protocol = botcore.PyFiBot
    allBots = None
    moduledir = os.path.join(sys.path[0], "modules/")
    startTime = None
    config = None

    def __init__(self, config):
        """Initialize the factory"""
        self.config = config
        self.data = {}
        self.data['networks'] = {}
        self.ns = {}
        # Cache url contents for 5 minutes, check for old entries every minute
        self._urlcache = timeoutdict.TimeoutDict(timeout=300, pollinterval=60)

    def startFactory(self):
        self.allBots = {}
        self.starttime = time.time()
        self._loadmodules()
        ThrottledClientFactory.startFactory(self)
        log.info("factory started")

    def stopFactory(self):
        del self.allBots
        ThrottledClientFactory.stopFactory(self)
        log.info("factory stopped")

    def buildProtocol(self, address):
        # we are connecting to a server, don't know which yet
        log.info("Building protocol for %s", address)

        # Go through all defined networks
        for network, server in self.data['networks'].items():
            log.debug("Looking for matching network: %s - %s", server, address)
            # get all of the ipv4 and ipv6 addresses configured for this domain name
            addrinfo = socket.getaddrinfo(server.address[0], server.address[1])
            ips = set()
            for ip in addrinfo:
                ips.add(ip[4][0]) # (2, 1, 6, '', ('192.168.191.241', 6667))

            # if the address we are connecting to matches one of the IP defined for
            # this network, connect to it and stop looking
            if address.host in ips:
                log.debug("Connecting to %s / %s", server, address)
                p = self.protocol(server)
                self.allBots[server.alias] = p
                p.factory = self
                return p

        # TODO: Remove this handling alltogether
        log.debug("Fall back to old process...")
        fqdn = socket.getfqdn(address.host)
        log.debug("Address: %s - %s", address, fqdn)
        # Fallback to the old, stupid, way of connecting
        for network, server in self.data['networks'].items():
            log.debug("Looking for matching network: %s - %s", server, fqdn)
            found = False
            if server.address[0] == fqdn:
                log.debug("fqdn matches server address")
                found = True
            if server.address[0] == address.host:
                log.debug("host matches server address")
                found = True
            if found:
                log.debug("Connecting to %s / %s", server, address)
                p = self.protocol(server)
                self.allBots[server.alias] = p
                p.factory = self
                return p

        # No address found
        log.error("Unknown network address: " + repr(address))
        return InstantDisconnectProtocol()

    def createNetwork(self, address, alias, nickname, channels=None, linerate=None, password=None, is_ssl=False):
        self.setNetwork(Network("data", alias, address, nickname, channels, linerate, password, is_ssl))

    def setNetwork(self, net):
        nets = self.data['networks']
        nets[net.alias] = net
        self.data['networks'] = nets

    def clientConnectionLost(self, connector, reason):
        """Connection lost for some reason"""
        log.info("connection to %s lost" % str(connector.getDestination().host))

        # find bot that connects to the address that just disconnected
        for n in self.data['networks'].values():
            dest = connector.getDestination()
            if (dest.host, dest.port) == n.address:
                if n.alias in self.allBots:
                    # did we quit intentionally?
                    if not self.allBots[n.alias].hasQuit:
                        # nope, reconnect
                        ThrottledClientFactory.clientConnectionLost(self, connector, reason)
                    del self.allBots[n.alias]
                    return
                else:
                    log.info("No active connection to known network %s" % n.address[0])

    def _finalize_modules(self):
        """Call all module finalizers"""
        for module in self._findmodules():
            # if rehashing (module already in namespace), finalize the old instance first
            if module in self.ns:
                if 'finalize' in self.ns[module][0]:
                    log.info("finalize - %s" % module)
                    self.ns[module][0]['finalize']()

    def _loadmodules(self):
        """Load all modules"""
        self._finalize_modules()
        for module in self._findmodules():
            env = self._getGlobals()
            log.info("load module - %s" % module)
            # Load new version of the module
            execfile(os.path.join(self.moduledir, module), env, env)
            # Initialize module
            if 'init' in env:
                log.info("initialize module - %s" % module)
                env['init'](self)
            # Add to namespace so we can find it later
            self.ns[module] = (env, env)

    def _findmodules(self):
        """Find all modules"""
        modules = [m for m in os.listdir(self.moduledir) if m.startswith("module_") and m.endswith(".py")]
        return modules

    def _getGlobals(self):
        """Global methods for modules"""
        g = {}

        g['getUrl'] = self.getUrl
        g['getNick'] = self.getNick
        g['isAdmin'] = self.isAdmin
        return g

    def getUrl(self, url, nocache=False):
        """Gets data, bs and headers for the given url, using the internal cache if necessary"""
        # work around urllib bug, don't send fragment to server <http://bugs.python.org/issue8280>
        url = urllib.splittag(url)[0]

        if url in self._urlcache and not nocache:
            log.info("cache hit : %s" % url)
        else:
            if nocache:
                log.info("cache pass: %s" % url)
            else:
                log.info("cache miss: %s" % url)
            self._urlcache[url] = URLCacheItem(url)
        return self._urlcache[url]

    def getNick(self, user):
        """Parses nick from nick!user@host
        @type user: string
        @param user: nick!user@host
        @return: nick"""
        return user.split('!', 1)[0]

    def isAdmin(self, user):
        """Check if an user has admin privileges.
        @return: True or False"""
        for pattern in self.config['admins']:
            if fnmatch.fnmatch(user, pattern):
                return True
        return False


def create_example_conf():
    """Create an example configuration file"""
    conf = """
    nick: botnick

    admins:
      - 'foo!bar@example.com'

    networks:
      ircnet:
        server: irc.ircnet.com
        channels:
          - mychannel
        linerate: 3
      quakenet:
        server: irc.quakenet.org
        authname: name
        authpass: password
        channels:
          - channel, password
    """

    examplefile = 'bot.config.example'
    if os.path.exists(examplefile):
        return False
    else:
        f = file(examplefile, 'w')
        yaml.dump(yaml.load(conf), f, default_flow_style=False)
        f.close()
        return True


def init_logging():
    filename = os.path.join(sys.path[0], 'pyfibot.log')
    # get root logger
    logger = logging.getLogger()
    if False:
        handler = logging.handlers.RotatingFileHandler(filename, maxBytes=5000 * 1024, backupCount=20)
    else:
        handler = logging.StreamHandler()
    # time format is same format of strftime
    formatter = logging.Formatter('%(asctime)-15s %(levelname)-8s %(name)-11s %(message)s')
    handler.setFormatter(formatter)
    logger.addHandler(handler)
    logger.setLevel(logging.DEBUG)

if __name__ == '__main__':
    init_logging()
    sys.path.append(os.path.join(sys.path[0], 'lib'))
    config = os.path.join(sys.path[0], "bot.config")

    if os.path.exists(config):
        config = yaml.load(file(config))
    else:
        if create_example_conf():
            print "No config file found, I created an example config (bot.config.example) for you. Please edit it and rename to bot.config."
        else:
            print 'No config file found, there is an example config (bot.config.example) for you. Please edit it and rename to bot.config or delete it to generate a new example config.'
        sys.exit(1)

    factory = PyFiBotFactory(config)
    for network, settings in config['networks'].items():
        # settings = per network, config = global
        nick = settings.get('nick', None) or config['nick']
        linerate = settings.get('linerate', None) or config.get('linerate', None)
        password = settings.get('password', None)
        is_ssl = bool(settings.get('is_ssl', False))
        port = int(settings.get('port', 6667))
        # normalize channel names to prevent internal confusion
        chanlist = []
        for channel in settings['channels']:
            if channel[0] not in '&#!+':
                channel = '#' + channel
            chanlist.append(channel)
        # TODO: this might cause issues, but bypassing it for now
        # resolve server name here in case it's a round-robin address
        #server_name = socket.getfqdn(settings['server'])
        server_name = settings['server']

        factory.createNetwork((server_name, port), network, nick, chanlist, linerate, password, is_ssl)
        if is_ssl:
            log.info("connecting via SSL to %s:%d" % (server_name, port))
            reactor.connectSSL(server_name, port, factory, ssl.ClientContextFactory())
        else:
            log.info("connecting to %s:%d" % (server_name, port))
            reactor.connectTCP(server_name, port, factory)
    reactor.run()



##
## ODB2 - Pure Python object database
## Copyright (C) 2004 Sami Hangaslammi (shang@jyu.fi)
##

import weakref
import gc
import time
import copy
import cPickle

import filestr

class ODBError(Exception): pass
class NotInitialised(ODBError): pass
class AlreadyInitialised(ODBError): pass
class NoSuchRoot(ODBError): pass
class InvalidObject(ODBError): pass
class RestoreError(ODBError): pass
class NoTransaction(ODBError): pass
error = ODBError ## so you can catch odb.error

from trdict import TrackingDict

## --- option flags ---
_auto_wake = True
_auto_transaction = False
_use_data_cache = True
_use_object_cache = False

_storage = None
_storage_cache = TrackingDict()
_object_cache = TrackingDict()
_roots = {}

_transaction = None ## current transaction
_transactions = weakref.WeakKeyDictionary()
_pickle_proto = -1

_debug_info = 0

_allocblock = 256

def open(filename, pickle_proto=None):
    global _storage, _roots, _pickle_proto
    if pickle_proto is not None:
        _pickle_proto = pickle_proto
    if _storage is not None:
        raise AlreadyInitialised
    _storage = filestr.ObjectStorage(filename, _allocblock)
    _storage.pickle_proto = _pickle_proto
    if not _storage.has_id(0):
        _roots = {}
        id = _storage.new(_roots)
        assert id == 0
    else:
        _roots = _storage[0]

def set_debug(flag):
    global _debug_info
    _debug_info = flag

def set_auto_wake(flag):
    global _auto_wake
    _auto_wake = flag

def set_auto_transaction(flag):
    global _auto_transaction
    _auto_transaction = flag

def set_data_cache(flag):
    global _use_data_cache
    _use_data_cache = flag

def set_object_cache(flag):
    global _use_object_cache
    _use_object_cache = flag

def set_flags(debug=None, auto_wake=None, auto_transaction=None, data_cache=None, object_cache=None):
    if debug is not None: set_debug(debug)
    if auto_wake is not None: set_auto_wake(auto_wake)
    if auto_transaction is not None: set_auto_transaction(auto_transaction)
    if data_cache is not None: set_data_cache(data_cache)
    if object_cache is not None: set_object_cache(object_cache)

def transaction():
    return _transaction

def new_transaction():
    global _transaction
    t = _transaction = ODBTransaction()
    return t

def clear_transaction():
    global _transaction
    _transaction = None

def clear_cache(timelimit=0):
    if _use_data_cache:
        _storage_cache.del_old(timelimit)
    if _use_object_cache:
        _object_cache.del_old(timelimit)

def close():
    global _storage, _roots
    if _storage is None:
        raise NotInitialised
    _roots = {}
    _storage.close()
    _storage = None

class TransactionError(ODBError): pass
class CommitConflict(TransactionError): pass

class ODBTransaction(object):

    def __new__(cls):
        o = object.__new__(cls)
        _transactions[o] = None
        if _debug_info:
            print "new transaction %s" % o
        return o

    def __init__(self):
        self._cache = {} ## id -> proxy/object
        self._commiting = False
        self._count = 0
        self._aborted = False
        self._new_roots = {}
        self._read_roots = {}

        self._write_buffer = {}

        ## actions made by other transactions during this transaction
        self._other_reads = {}
        self._other_writes = {}
        self._other_roots = {}

    def __getattribute__(self, key):
        ## EXPERIMENTAL: for automatically setting self as the current transaction when methods are accessed        
        global _transaction
        _transaction = self
        return object.__getattribute__(self, key)

    def activate(self):
        """transaction.activate() -> None

        Activate this transaction.
        """
        global _transaction
        if self._aborted:
            raise TransactionError, "Cannot activate a finished transaction"
        if self._commiting:
            raise TransactionError, "Cannot activate a transaction that is commiting data"
        _transaction = self

    def commit(self):
        """transaction.commit() -> int

        Returns the number of objects updated in the commit. The method will
        throw CommitConflict if another transaction has modified the same
        objects.
        """
        global _transaction
        if self._aborted:
            raise TransactionError, "Cannot commit a finished transaction"
        if self._commiting:
            raise TransactionError, "Already commiting this transaction"
        if _storage is None:
            raise NotInitialised

        if _debug_info:
            print "committing transaction %s" % self

        try:
            self._commiting = True
            _transaction = self

            deletions = {}
            reads = {}
            writes = {}

            ## Check for conflicts
            for k,v in self._cache.items():
                if not isinstance(v, ODBProxy) and k in self._other_writes:
                    raise CommitConflict
            for k,v in self._new_roots.items():
                if k in self._other_roots:
                    raise CommitConflict
            for k in self._read_roots:
                if k in self._other_roots:
                    raise CommitConflict

            ## Write updated objects and mark read objects
            for k,v in self._cache.items():
                if not isinstance(v, ODBProxy):
                    id = v._p_id
                    if v._p_deleted:
                        writes[id] = None
                        deletions[id] = None
                    elif v._p_changed:
                        writes[id] = None
                        self._store_object(v)
                    else:
                        reads[id] = None

            new_roots = {}
            ## Validate new roots
            for k,v in self._new_roots.iteritems():
                id = self._flatten_object(v)
                new_roots[k] = id

        except:
            self._commiting = False
            raise

        ## Everything has been pickled succesfully. Now do the real thing.

        ## Update the status of other transactions
        for t in _transactions:
            if t is not self:
                t._other_reads.update(reads)
                t._other_writes.update(writes)
                t._other_roots.update(new_roots)

        ## Remove deleted items
        for id in deletions:
            if id in _storage:
                del _storage[id]
                self._count += 1
            if _use_data_cache and id in _storage_cache: ## TODO: faster to try-catch
                del _storage_cache[id]
            if _use_object_cache and id in _object_cache:
                del _object_cache[id]

        ## Write updated objects
        for id, pckl in self._write_buffer.iteritems():
            if id in _storage:
                if _debug_info:
                    print "writing object %i to file" % id
                _storage.set_pickle(id, pckl)
                if _use_data_cache: _storage_cache[id] = pckl
                self._count += 1
            self._flatten_object(self._cache[id])

        ## Write new roots
        if new_roots:
            _roots.update(new_roots)
            _storage[0] = _roots
            

        _storage.flush()

        count = self._count
        self._aborted = True
        self._commiting = False
        del _transactions[self]
        if _transaction is self:
            _transaction = None
        return count
                    

    def restart(self):
        """transaction.restart() -> None

        Cancel all changes and restart the transaction. All objects waken
        by this transaction are put back to hibernation and must be reawakened.
        """
        if self._aborted:
            raise TransactionError, "Cannot restart a finished transaction"
        self.__init__() ## TODO

    def abort(self):
        """transaction.abort() -> None

        Cancel all changes and invalidate this transaction.
        """
        global _transaction
        self.__init__() ## TODO
        del _transactions[self]
        self._aborted = True
        if _transaction is self:
            _transaction = None
        
    ### --- ###

    def get_roots(self):
        """transaction.get_roots() -> [list of root keys]"""
        return _roots.keys() ## TODO

    def get_root(self, name=""):
        """transaction.get_root([name]) -> root

        Return a root object of the database and awaken it.
        """
        if name in self._new_roots:
            return self._new_roots[name]
        else:
            if name not in _roots:
                raise NoSuchRoot, name
            id = _roots[name]
        self._read_roots[name] = None
        if name in self._other_roots:
            del self._other_roots[name]
        proxy = self._restore_object(id)
        return self.wake(proxy)
        

    def set_root(self, name, obj=None):
        """transaction.set_root([name,] object) -> None

        Assign a root object for the database. Effective after commit.
        """
        if obj is None:
            obj = name
            name = ""
        self.wake(obj)
        if not isinstance(obj, (ODBObject, ODBProxy)):
            raise TypeError, "root objects must be instances of ODBObject"
        if _storage is None:
            raise NotInitialised
        self._new_roots[name] = obj

    def mark_changed(self, obj):
        """transaction.mark_changed(object) -> None

        Mark an object as "changed" so that it will be written to the
        database when committing the transaction. Usually ODBObjects can
        keep track of this automatically, but this method might be needed
        if an object has non-odb-aware mutable attributes.
        """
        global _transaction
        _transaction = self
        obj.__odb_changed__()

    def wake(self, *objs):
        """transaction.wake(object[,object2][,object3]...) -> object

        Awaken a proxy object so that it can be used in operations.
        """
        global _transaction
        _transaction = self
        for obj in objs:
            if not isinstance(obj, ODBProxy):
                continue
            self._restore_proxy(obj)
        if len(objs) == 1:
            return objs[0]
        return objs

    def delete(self, obj):
        """transaction.delete(object) -> None
        Cause an object to be deleted from database when this
        transaction is committed.
        """        
        global _transaction
        _transaction = self
        if not isinstance(obj, (ODBProxy, ODBObject)):
            raise TypeError, "cannot remove a non-odb object"
        self.wake(obj)
        if obj._p_deleted:
            return
        obj.__odb_changed__()
        obj._p_deleted = True

    def _reduce_object(self, obj):
        if not isinstance(obj, ODBObject):
            raise TypeError, "can only reduce isntances of ODBObject"
        if obj._p_id not in self._cache:
            raise TransactionError, "object %r not owned by transaction %r" % (obj, self)
        dct = obj.__get_odb_state__()
        cls = obj.__class__
        if _use_object_cache and obj._p_id in _object_cache:
            _object_cache[obj._p_id] = (cls,dct)
        return (cls,dct)

    def _pickle(self, obj):
        reduced = self._reduce_object(obj)
        return cPickle.dumps(reduced, _pickle_proto)
        
    def _new_object(self, obj):
        assert self._commiting
        if not isinstance(obj, ODBObject):
            raise TypeError, "can only register isntances of ODBObject with the ODB"
        if _storage is None:
            raise NotInitialised
        if obj._p_deleted:
            return
        if "_p_id" in obj.__dict__:
            return obj._p_id
        id = _storage.new_id()
        if _debug_info:
            print "new object %r gets id %i" % (obj, id)
        obj._p_id = id
        obj._p_changed = True
        self._cache[id] = obj
        self._write_buffer[id] = self._pickle(obj)
        return id

    def _store_object(self, obj):
        assert self._commiting
        if isinstance(obj, ODBProxy):
            return obj._p_id
        if not isinstance(obj, ODBObject):
            raise InvalidObject, "Cannot store object of type %s" % type(obj)
        if obj._p_deleted:
            return
        if "_p_id" in obj.__dict__:
            id = obj._p_id
            if not ('p_changed' in obj.__dict__) or obj._p_changed:
                if _debug_info:
                    print "putting object %i to write buffer" % id
                self._write_buffer[id] = self._pickle(obj)
            else:
                if _debug_info:
                    print "object %i hasn't changed, not saving" % id
            obj._p_changed = False
            return id
        return self._new_object(obj)
        
    def _flatten_object(self, obj):
        assert self._commiting
        if isinstance(obj, ODBProxy):
            return obj._p_id
        if obj._p_deleted:
            return -1
        if not isinstance(obj, ODBObject):
            raise InvalidObject, "Cannot flatten object of type %s" % type(obj)
        self._store_object(obj)
        id = obj._p_id
        if _debug_info:
            print "flatting object %i to a proxy" % id
        obj.__clear_odb_state__()
        obj.__dict__.clear()    
        obj.__dict__.update({'_p_id':id})
        obj.__class__ = ODBProxy
        return id

    def _load_state(self, id):
        if _use_data_cache:
            if id in _storage_cache: ## TODO: probably faster with try-catch
                pckl = _storage_cache[id]
            else:            
                pckl = _storage.get_pickle(id)
                _storage_cache[id] = pckl
        else:
            pckl = _storage.get_pickle(id)            
            
        ##cls,dct = _storage[id]
        return cPickle.loads(pckl)

    def _restore_proxy(self, proxy):
        if not isinstance(proxy, ODBProxy):
            raise TypeError, "tried to restore a non-proxy object"        
        id = proxy._p_id
        if id in self._cache and not isinstance(self._cache[id], ODBProxy):
            if _debug_info:
                print "getting object %i from transaction cache" % id
            return self._cache[id]
        if _debug_info:
            print "restoring object %i" % id

        if _use_object_cache and id in _object_cache:
            if _debug_info:
                print "...from object cache"
            cls,dct = _object_cache[id]
        else:
            cls,dct = self._load_state(id)

        obj = cls.__new__(cls)
        obj.__set_odb_state__(dct)        
        obj.__dict__['_p_changed'] = 0
        obj.__dict__['_p_id'] = id
        obj.__dict__['_p_deleted'] = False
        self._cache[id] = obj
        if _use_object_cache:
            _object_cache[id] = cls,dct
        if _debug_info:
            print "...got object %r" % obj
        if id in self._other_writes:
            ## object read after another transaction commited it, so it is up-to-date
            del self._other_writes[id]
        return obj

    def _restore_object(self, id):
        global _transaction
        _transaction = self
        return _restore_object(id)


def _r(id):
    if _storage is None:
        raise NotInitialised
    if _transaction is None:
        if not _auto_transaction:
            raise NoTransaction
        new_transaction()
    if id not in _storage:
        if _debug_info:
            print "Object %i has been deleted!" % id
        return None
##    try:
##        obj = _transaction._cache[id]
##        if _debug_info:
##            print "restoring object %i from cache" % id        
##    except KeyError:
    if _debug_info:
        print "creating proxy for object %i" % id
    obj = ODBProxy()
    obj._p_id = id
    if id not in _transaction._cache:
        _transaction._cache[id] = obj
    return obj
_restore_object = _r
_r.__safe_for_unpickling__ = 1

def _none():
    return None
_none.__safe_for_unpickling__ = 1


class ODBProxy(object):
    _members = '_p_id', '__getstate__','__setstate__','__reduce__','_p_restore','__reduce_ex__','__class__'

    def _p_restore(self):
        if _transaction is None:
            if not _auto_transaction:
                raise NoTransaction
            new_transaction()
        if not _auto_wake and (self._p_id not in _transaction._cache \
           or isinstance(_transaction._cache[self._p_id], ODBProxy)):
            raise ODBError, "Accessing an hibernating object. Call wake first"
        return _transaction._restore_proxy(self)

    def __reduce__(self):
        return (_r, (self._p_id,))

    def __getattribute__(self, name):
        if name in ODBProxy._members:
            return object.__getattribute__(self, name)
        if _debug_info:
            print "getting attribute %s from proxy" % name
        state = self._p_restore()
        return getattr(state, name)

    def __setattr__(self, name, value):
        if name in ODBProxy._members:
            object.__setattr__(self,name,value)
            return
        state = self._p_restore()
        setattr(state, name, value)

    def __delattr__(self, name):
        state = self._p_restore()
        delattr(state, name)

    def __hasattr__(self, name):
        state = self._p_restore()
        return hasattr(state, name)

    def __getitem__(self, name):
        state = self._p_restore()
        return state.__getitem__(name)

    def __setitem__(self, name, value):
        state = self._p_restore()
        state.__setitem__(name, value)

    def __contains__(self, name):
        state = self._p_restore()
        return state.__contains__(name)

    def __delitem__(self, name):
        state = self._p_restore()
        state.__delitem__(name)

    def __iter__(self):
        state = self._p_restore()
        return iter(state)

    def __hash__(self):
        state = self._p_restore()
        return hash(state)

    def __str__(self):
        state = self._p_restore()
        return str(state)

    def __repr__(self):
        state = self._p_restore()
        return repr(state)

    def __cmp__(self, other):
        state = self._p_restore()
        return cmp(state, other)

    def __len__(self):
        state = self._p_restore()
        return len(state)
   
class ODBObject(object):
    _p_deleted = False
    _p_changed = False
    
    def __reduce__(self):
        #print "reducing %r" % self
        if '_p_deleted' in self.__dict__ and self._p_deleted:
            return (_none, ())
        if _transaction is None:
            raise NoTransaction
        if not '_p_id' in self.__dict__:
            _transaction._new_object(self)
            #print "new object:%s" % self._p_id
        else:
            #print "old object:%s" % self._p_id
            if self._p_id not in _transaction._cache:
                raise TransactionError, "object in wrong transaction"
        return (_r, (self._p_id,))

    def __get_odb_state__(self):
        d = {}
        d.update(self.__dict__)
        if '_p_id' in d: del d['_p_id']
        if '_p_changed' in d: del d['_p_changed']
        if '_p_deleted' in d: del d['_p_deleted']
        return d

    def __set_odb_state__(self, state):
        self.__dict__.clear()
        self.__dict__.update(state)

    def __clear_odb_state__(self):
        pass

    def __odb_changed__(self):
        ## TODO: copy-on-write
        if self._p_changed: return
        if _use_object_cache and "_p_id" in self.__dict__ and self._p_id in _object_cache:
            if _transaction is None:
                if not _auto_transaction:
                    raise NoTransaction
                new_transaction()
                _transaction._cache[self._p_id] = self
            cls,dct = _transaction._load_state(self._p_id)
            id = self._p_id
            deleted = self._p_deleted
            self.__set_odb_state__(dct)
            self.__dict__['_p_id'] = id
            self.__dict__['_p_deleted'] = deleted
        object.__setattr__(self, "_p_changed", True)

    def __setitem__(self, key, value):
        self.__odb_changed__()
        object.__setitem__(self, key, value)

    def __setattr__(self, key, value):
        if key.startswith("_p_") or key.startswith("__"):
            pass
        else:
            self.__odb_changed__()
        object.__setattr__(self, key, value)
  
    def __delitem__(self, key):
        self.__odb_changed__()
        object.__delitem__(self, key)

    def __delattr__(self, key):
        self.__odb_changed__()
        object.__delattr__(self, key)
        
    def type(self):
        return type(self)

    def isinstance(self, cls):
        return isinstance(self, cls)

class _NoParam(object): pass

class ODBList(ODBObject):
    def __init__(self, items=None):
        if items is None:
            self.list = []
        else:
            self.list = list(items)
    
    def __setitem__(self, index, value):
        self.__odb_changed__()
        self.list[index] = value
    
    def __delitem__(self, index):
        self.__odb_changed__()
        del self.list[index]

    def insert(self, index, obj):
        self.__odb_changed__()
        self.list.insert(index, obj)

    def append(self, obj):
        self.__odb_changed__()
        self.list.append(obj)

    def pop(self, index=None):
        self.__odb_changed__()
        if index is None:
            return self.list.pop()
        self.list.pop(index)

    def sort(self):
        self.__odb_changed__()
        self.list.sort()

    def __getitem__(self, index):
        return self.list[index]

    def __iter__(self):
        return iter(self.list)

    def __contains__(self, item):
        return item in self.list

    def __len__(self):
        return len(self.list)

class ODBDict(ODBObject):
    
    def __init__(self, *arg):
        self.dict = dict(*arg)
    
    def __setitem__(self, key, value):
        self.__odb_changed__()
        self.dict[key] = value
    
    def __delitem__(self, key):
        self.__odb_changed__()
        del self.dict[key]

    def setdefault(self, key, value):
        self.__odb_changed__()
        self.dict.setdefault(key, value)
    
    def clear(self):
        self.__odb_changed__()
        self.dict.clear(self)

    def update(self, src):
        self.__odb_changed__()
        self.dict.update(self, src)

    def get(self, key, default=_NoParam):
        if default is _NoParam:
            return self.dict.get(key)
        return self.dict.get(key, default)

    def pop(self, key, default=_NoParam):
        self.__odb_changed__()
        if default is _NoParam:
            return self.dict.pop(key)
        return self.dict.pop(key, default)
  
    def __getitem__(self, key):
        return self.dict[key]

    def __iter__(self):
        return iter(self.dict)

    def __contains__(self, key):
        return key in self.dict

    def __len__(self):
        return len(self.dict)

    def items(self):
        return self.dict.items()

    def keys(self):
        return self.dict.keys()

    def values(self):
        return self.dict.values()

    def iteritems(self):
        return self.dict.iteritems()

    def iterkeys(self):
        return self.dict.iterkeys()

    def itervalues(self):
        return self.dict.itervalues()

class ODBObjectVersioned(ODBObject):
    persistenceVersion = 0

    def __get_odb_state__(self):
        d=ODBObject.__get_odb_state__(self)
        d['persistenceVersion']=self.__class__.persistenceVersion

        print '__get_odb_state__', d
        
        return d

    def __set_odb_state__(self, state):
        ODBObject.__set_odb_state__(self, state)

        curVer=state.get('persistenceVersion', 0)

        ## print 'preVer', curVer, self.__class__.persistenceVersion

        if curVer < self.__class__.persistenceVersion:
            while curVer < self.__class__.persistenceVersion:
                curVer += 1
                method=self.__class__.__dict__.get('upgradeToVersion%d' % curVer, None)
                if method:
                    method(self)



import os
from paver.easy import *
import paver.virtual
import paver.setuputils
from paver import svn
from paver.setuputils import setup, find_package_data, find_packages

options(
    #  bootstrap=Bunch(bootstrap_dir="bootstrap"),
      virtualenv=Bunch(
              packages_to_install=[],
              install_paver = True,
              paver_command_line=None,
              no_site_packages=True
          )
    )

install_requires = ['twisted', 'requests', 'pyyaml', 'imdbpy', 'beautifulsoup']

setup(
      name='pyfibot',
      version='0.9.1',
      description='An event-based IRC bot, based on twisted.protocols.irc.IRCClient',
      url='http://code.google.com/p/pyfibot/',
      author='Riku Lindblad',
      author_email='riku.lindblad@gmail.com',
      license='MIT',
      packages=['pyfibot'],
      zip_safe=False,
      install_requires=install_requires
      )

@task
def prepare(options):
      pass

@task
def bootstrap(options):
    """create virtualenv"""
    try:
        import virtualenv
    except ImportError, e:
        raise RuntimeError("virtualenv is needed for bootstrap")

    options.virtualenv.no_site_packages = False
    options.virtualenv.packages_to_install = install_requires
    options.bootstrap.no_site_packages = False
    options.virtualenv.paver_command_line='prepare'
    call_task('paver.virtual.bootstrap')



# -*- coding: utf-8 -*-

"""
$Id: module_nugget.py 340 2012-11-14 22:16:45Z riku.lindblad@gmail.com $
$HeadURL: https://pyfibot.googlecode.com/svn/trunk/pyfibot/modules/available/module_nugget.py $
"""

from urllib import FancyURLopener
from BeautifulSoup import BeautifulSoup
import re
import random


class MyOpener(FancyURLopener):
    version = 'Mozilla/5.0 (Windows; U; Windows NT 5.1; it; rv:1.8.1.11) Gecko/20071127 Firefox/2.0.0.11'


class Nuggets():
    def __init__(self):
        self.nugget_list = []
        self.getNuggets()
        self.soup = None
        random.seed()

    def makeSoup(self):
        myopener = MyOpener()
        page = myopener.open(self.url).read()
        self.soup = BeautifulSoup(page)

    def getNuggets(self):
        pass

    def cleanNuggets(self, ugly_nugget_list):
        for nugget in ugly_nugget_list:
            temp = re.sub('<.*?>', '', str(nugget))
            if re.search('\.\.\.', temp):
                self.nugget_list.append(re.sub('\.\.\.', '', temp))

    def getSentence(self):
        return self.question + self.nugget_list[random.randint(0, len(self.nugget_list))]


class NuggetsEn(Nuggets):
    def __init__(self, archive_num):
        self.question = "Did you know, "
        #archive_max checked 13-8-2009
        self.archive_max = 246
        self.archive_num = archive_num
        if self.archive_num > self.archive_max:
            self.archive_num = random.randint(1, self.archive_max)
        self.url = "http://en.wikipedia.org/wiki/Wikipedia:Recent_additions_" + str(self.archive_num)
        Nuggets.__init__(self)

    def getNuggets(self):
        self.makeSoup()
        ugly_nugget_list = []
        lists = self.soup.findAll("ul")
        for list in lists[:-7]:
            for nugget in list.findAll("li"):
                ugly_nugget_list.append(nugget)

        self.cleanNuggets(ugly_nugget_list)


class NuggetsFi(Nuggets):
    def __init__(self, archive_num):
        self.question = "Tiesitkö, että "
        #archive_max checked 13-8-2009
        self.archive_max = 6
        self.archive_num = archive_num
        if self.archive_num > self.archive_max:
            self.archive_num = random.randint(1, self.archive_max)
        self.url = "http://fi.wikipedia.org/wiki/Wikipedia:Tiesitk%C3%B6_ett%C3%A4.../Arkisto" + str(self.archive_num)
        Nuggets.__init__(self)

    def getNuggets(self):
        self.makeSoup()
        ugly_nugget_list = []
        lists = self.soup.findAll("ul")
        for nugget in lists[0].findAll("li"):
            ugly_nugget_list.append(nugget)

        self.cleanNuggets(ugly_nugget_list)


def command_nugget(bot, user, channel, args):
    """Tells nugget from Wikipedia. Avaible Languages: fi, en Usage: .nugget [lang]"""
    if args:
        if args.lower() == "en":
            nugget = NuggetsEn(10000)
        elif args.lower() == "fi":
            nugget = NuggetsFi(10000)
        else:
            nugget = NuggetsFi(10000)
    else:
        nugget = NuggetsFi(10000)
    return bot.say(channel, nugget.getSentence())



#!/usr/bin/python
# -*- coding: iso8859-1 -*-
## (c)2004 Timo Reunanen <parker _et_ wolfenstein _dit_ org>

import time
import re

_exact=r'''
^
(?P<hour> \d{1,2})    ## hour
[:.]
(?P<min> \d{2})       ## minutes
(?:
  [:.]
  (?P<sec>\d{2} )     ## secods (optional)
)?
$
'''

_add=r'''
^
[+]
(?:                   ## hour
  (?P<hour> \d+)+h    ## syntax: 1234h
)?                    ## optional
\s*
(?:                   ## minutes
  (?P<min> \d+)+m     ## syntax: 1234m
)?                    ## optional
\s*
(?:                   ## seconds
  (?P<sec> \d+)+s?    ## syntax: 1234s or 1234
)?                    ## optional
$
'''

exactRe=re.compile(_exact, re.VERBOSE | re.MULTILINE | re.I)
addRe=re.compile(_add, re.VERBOSE | re.MULTILINE | re.I)

class TimeException(Exception): pass

def convert(s):
    s=s.strip()

    m=exactRe.match(s)
    if m:
        tm=time.time()
        year, mon, mday, hour, min, sec, wday, yday, isdst = time.localtime(tm)

        hour=int(m.group('hour'))
        min=int(m.group('min'))
        sec=int(m.group('sec') or '00')

        ret=time.mktime( (year, mon, mday, hour, min, sec, wday, yday, isdst) )

        while ret < tm:
            ret += 86400

        return ret

    m=addRe.match(s)
    if m:
        hour=int(m.group('hour') or '0')
        min=int(m.group('min') or '0')
        sec=int(m.group('sec') or '0')

        addSecs=hour*3600 + min*60 + sec

        return time.time()+addSecs

    raise TimeException('Invalid syntax')

if __name__=='__main__':
    year, mon, mday, hour, min, sec, wday, yday, isdst = time.localtime()
    print (hour, min, sec)
    print time.time()-time.mktime(time.localtime())
    
    print convert('11.23')-time.time()
    




import time
from threading import Thread, RLock

class _CleanupThread(Thread):
    def __init__(self, d, pollinterval):
        Thread.__init__(self)
        self.d = d
        self.interval = pollinterval
        self.setDaemon(True)
        self.running = True

    def run(self):
        while self.running:
            time.sleep(self.interval)
            self.d.cleanup()

    def stop(self):
        self.running = False

class TimeoutDict(dict):
    def __init__(self, timeout=0, pollinterval=10.0):
        dict.__init__(self)
        self._timeout = timeout
        self._lastchange = {}
        self._lock = RLock()

        if timeout > 0:
            self._thread = _CleanupThread(self, pollinterval)
            self._thread.start()
        else:
            self._thread = None

    def __setitem__(self, key, value):
        self._lock.acquire()
        try:
            dict.__setitem__(self, key, value)
            self._lastchange[key] = time.time()
        finally:
            self._lock.release()

    def update(self, src):
        for k,v in src.iteritems():
            self[k] = v

    def __delitem__(self, key):
        self._lock.acquire()
        try:
            dict.__delitem__(self, key)
            del self._lastchange[key]
        finally:
            self._lock.release()

    def __del__(self):
        if self._thread is not None:
            self._thread.stop()

    def cleanup(self, timeout=None):
        self._lock.acquire()
        try:
            if timeout is None:
                timeout = self._timeout
            current = time.time()
            for k,t in self._lastchange.items():
                if current > t + timeout:
                    del self[k]
        finally:
            self._lock.release()



"""
$Id: module_autoop.py 331 2012-11-13 10:18:18Z riku.lindblad@gmail.com $
$HeadURL: https://pyfibot.googlecode.com/svn/trunk/pyfibot/modules/module_autoop.py $

Commands:
.autoop add/del/status/list
.op

Nick-format: nick!ident@example.org

Supports adding/deleting multiple autoop-rules:
.autoop add #testchannel firstNick!ident@example.org secNick!ident@example.org

In add/del #channel needed, if changes done in query. If changes are made in #channel, defaults to current #channel.

config-file in root/modules/module_autooop.conf
config-format:
 'nick!ident@example.org':
 - '#channelOne'
 - '#channelTwo'

 'secNick!ident@example.org':
 - '#channelTwo'
"""

import fnmatch
import yaml
import os.path
import sys
import re

oplistfile = os.path.join(sys.path[0], 'modules', 'module_autoop.conf')
oplist = dict()


def init(botconfig):
    global oplist
    if os.path.exists(oplistfile):
        oplist = yaml.load(file(oplistfile))


def writeConfig():
    # open file for writing (note, removes all markings that have been made to config-file prior to .rehash
    f = open(oplistfile, 'w')
    yaml.dump(oplist, f, default_flow_style=False)
    f.close()


def isAutoOppable(user, channel):
    for pattern in oplist:
        if fnmatch.fnmatch(user, pattern):
            if channel in oplist[pattern]:
                return True


def handle_userJoined(bot, user, channel):
    if isAdmin(user) or isAutoOppable(user, channel):
        nick = getNick(user)
        bot.log('auto-opping %s on %s' % (user, channel))
        bot.mode(channel, True, 'o', user=nick)


def handle_args(args, channel):
    message = ''
    nicks = list()
    rg = re.compile('.*?(!).*?(@).*?', re.IGNORECASE | re.DOTALL)    # regex for validating nick format
    if re.match(rg, args[0].strip()) is None and args[0].strip()[0] == '#':
        op_channel = args[0].strip()
        for nick in args[1:]:
            if re.match(rg, nick):
                nicks.append(nick)
            else:
                message += '%s not in correct format. ' % nick
    else:
        op_channel = channel
        for nick in args[0:]:
            if re.match(rg, nick):
                nicks.append(nick)
            else:
                message += '%s not in correct format. ' % nick
    return nicks, op_channel, message


def autoop_status(user, channel, args):
    nick = getNick(user)
    if isAdmin(user):
        return '%s: you are %s on %s, status: bot admin' % (nick, user, channel)
    elif isAutoOppable(user, channel):
        return '%s: you are %s on %s, status: auto-op' % (nick, user, channel)
    else:
        return '%s: you are %s on %s: status: -' % (nick, user, channel)


def autoop_add(user, channel, args):
    if isAdmin(user):
        nicks = list()
        message = ''
        if len(args) > 0:
            nicks, op_channel, message = handle_args(args, channel)
            if cmp(op_channel, user) != 0:
                for nick in nicks:
                    if nick in oplist:    # if already in oplist, just add to existing list
                        if op_channel in oplist[nick]:    # if not in nicks channel list, add
                            message += 'Auto-op for %s already in %s. ' % (nick, op_channel)
                        else:
                            message += 'Auto-op for %s in %s added. ' % (nick, op_channel)
                            oplist[nick].append(op_channel)
                    else:
                        message += 'Auto-op for %s in %s added. ' % (nick, op_channel)
                        oplist[nick] = list()
                        oplist[nick].append(op_channel)
                writeConfig()
            else:
                message = '#channel always needed when making changes in query!'
            return message
        else:
            return 'Usage: .autoop add [#channel] nick!ident@hostma.sk'


def autoop_del(user, channel, args):
    if isAdmin(user):
        message = ''
        if len(args) > 0:
            nicks, op_channel, message = handle_args(args, channel)
            if cmp(op_channel, user) != 0:
                for nick in nicks:
                    if nick in oplist:
                        if op_channel in oplist[nick]:  # if in nicks channel list, remove
                            message += 'Auto-op for %s in %s removed. ' % (nick, op_channel)
                            oplist[nick].remove(op_channel)
                            if len(oplist[nick]) == 0:
                                del oplist[nick]
                        else:
                            message += 'No auto-op for %s in %s. ' % (nick, op_channel)
                    else:
                        message += 'No auto-op for %s in %s. ' % (nick, op_channel)
                writeConfig()
            else:
                message = '#channel always needed when making changes in query!'
            return message
        else:
            return 'Usage: .autoop del [#channel] nick!ident@hostma.sk'


def autoop_list(user, channel, args):
    if isAdmin(user):
        i = 0
        rg = re.compile('.*?(!).*?(@).*?', re.IGNORECASE | re.DOTALL)  # regex for validating nick format
        if (len(args) == 0 or (re.match(rg, args[0].strip()) is None and args[0].strip()[0] == '#')) and args[0] != 'me':
            if len(args) == 0:
                op_channel = channel
            elif re.match(rg, args[0].strip()) is None and args[0].strip()[0] == '#':
                op_channel = args[0].strip()
            message = 'Auto-ops in %s: ' % op_channel
            for nick, channels in oplist.items():
                for chan in channels:
                    if chan == op_channel:
                        message += nick + ' '
                        i = i + 1
            if i != 0:
                return message
            else:
                return '%s has no auto-ops.' % op_channel
        elif re.match(rg, args[0].strip()) or args[0] == 'me':
            if args[0] != 'me':
                nick = args[0].strip()
            else:
                nick = user
            message = '%s auto-op in: ' % nick
            if nick in oplist:
                for chan in oplist[nick]:
                    message += chan + ' '
                    i = i + 1
            if i != 0:
                return message
            else:
                return '%s not auto-op anywhere.' % nick
        else:
            return 'Usage: .autoop_list [#channel _or_ nick!ident@hostma.sk]'
    else:
        nick = user
        i = 0
        message = '%s auto-op in: ' % nick
        if nick in oplist:
            for chan in oplist[nick]:
                message += chan + ' '
                i = i + 1
        if i != 0:
            return message
        else:
            return '%s not auto-op anywhere.' % nick


def make_valid_commands_msg(commands):
    message = 'Valid commands are: '
    for command in commands:
        message += command
        message += ', '
    message = message[:-2]
    message += '.'
    return message


def command_op(bot, user, channel, args):
    if isAdmin(user) or isAutoOppable(user, channel):
        nick = getNick(user)
        bot.log('opping %s by request on %s' % (user, channel))
        bot.mode(channel, True, 'o', user=nick)


def command_autoop(bot, user, channel, args):
    commands = {'add': autoop_add, 'del': autoop_del, 'list': autoop_list, 'status': autoop_status}
    args = args.split()
    if len(args) > 0:
        command = args[0]
        if command in commands:
            args.remove(command)
            message = commands[command](user, channel, args)
        else:
            message = make_valid_commands_msg(commands)
    else:
        message = make_valid_commands_msg(commands)
    return bot.say(channel, message)



# Finnish name days by chiman 9/2004

import time

namedays = []
# January
namedays.append(['-', 'Aapeli', 'Elmo, Elmer', 'Ruut', 'Lea, Leea', 'Harri',
                 'Aukusti, Aku, August', 'Hilppa, Titta',
                 'Veikko, Veli, Veijo', 'Nyyrikki', 'Kari, Karri', 'Toini',
                 'Nuutti', 'Sakari, Saku', 'Solja', 'Ilmari, Ilmo',
                 'Toni, Anttoni, Antto, Anton', 'Laura',
                 'Heikki, Henrik, Henrikki, Henri', 'Henna, Henni, Henriikka',
                 'Aune, Oona, Auni', 'Visa', 'Eine, Eini, Enni', 'Senja',
                 'Paavo, Pauli, Paavali, Paul', 'Joonatan', 'Viljo',
                 'Kaarlo, Kalle, Kaarle, Mies', 'Valtteri', 'Irja', 'Alli'])
# February
namedays.append(['Riitta', 'Aamu', 'Valo', 'Armi', 'Asser',
                 'Terhi, Teija, Tiia, Tea, Terhikki', 'Riku, Rikhard',
                 'Laina', 'Raija, Raisa', 'Elina, Elna, Ella, Ellen',
                 'Talvikki', 'Elma, Elmi', 'Sulo, Sulho', 'Voitto, Tino',
                 'Sipi, Sippo', 'Kai', 'Väinö, Väinämö', 'Kaino', 'Eija',
                 'Heli, Helinä, Heljä, Hely', 'Keijo',
                 'Tuulikki, Tuuli, Tuulia', 'Aslak', 'Matti, Mattias',
                 'Tuija, Tuire', 'Nestori', 'Torsti', 'Onni', '-'])
# March
namedays.append(['Alpo, Alvi, Alpi', 'Virve, Virva', 'Kauko',
                 'Ari, Arsi, Atro', 'Laila, Leila', 'Tarmo', 'Tarja, Taru',
                 'Vilppu', 'Auvo', 'Aurora, Aura, Auri', 'Kalervo',
                 'Reijo, Reko', 'Erno, Tarvo, Ernesti', 'Matilda, Tilda',
                 'Risto', 'Ilkka', 'Kerttu, Kerttuli', 'Eetu, Edvard',
                 'Jooseppi, Juuso, Joosef, Josefiina',
                 'Aki, Joakim, Kim, Jaakkima', 'Pentti', 'Vihtori', 'Akseli',
                 'Kaapo, Gabriel, Kaapro, Kaappo', 'Aija',
                 'Manu, Immanuel, Manne, Immo', 'Sauli, Saul', 'Armas',
                 'Joonas, Jouni, Joni, Joona, Jonne, Jonni', 'Usko, Tage',
                 'Irma, Irmeli'])
# April
namedays.append(['Raita, Pulmu', 'Pellervo', 'Sampo', 'Ukko',
                 'Irene, Irina, Ira, Iro',
                 'Vilho, Ville, Viljami, Vilhelm, Vili', 'Allan, Ahvo',
                 'Suoma, Suometar', 'Elias, Eelis, Eljas', 'Tero', 'Verna',
                 'Julius, Julia', 'Tellervo', 'Taito', 'Linda, Tuomi',
                 'Jalo, Patrik', 'Otto', 'Valto, Valdemar', 'Pälvi, Pilvi',
                 'Lauha', 'Anssi, Anselmi', 'Alina',
                 'Yrjö, Jyrki, Jyri, Yrjänä, Jori', 'Pertti, Albert, Altti',
                 'Markku, Markus, Marko', 'Terttu, Teresa', 'Merja',
                 'Ilpo, Ilppo, Tuure', 'Teijo',
                 'Mirja, Mirva, Mira, Miia, Mirjami, Mirka, Mirkka'])
# May
namedays.append(['Vappu, Valpuri', 'Vuokko, Viivi', 'Outi', 'Ruusu, Roosa',
                 'Maini', 'Ylermi', 'Helmi, Kastehelmi', 'Heino', 'Timo',
                 'Aino, Aina, Aini, Ainikki', 'Osmo', 'Lotta',
                 'Kukka, Floora', 'Tuula', 'Sofia, Sonja, Sohvi',
                 'Esteri, Essi, Ester', 'Maila, Maili, Mailis',
                 'Erkki, Eero, Eerikki',
                 'Emilia, Milja, Emma, Emmi, Milla, Amalia',
                 'Lilja, Karoliina, Lilli', 'Kosti, Kosta, Konstantin',
                 'Hemminki, Hemmo', 'Lyydia, Lyyli', 'Tuukka, Touko',
                 'Urpo', 'Minna, Vilma, Vilhelmiina, Mimmi', 'Ritva',
                 'Alma', 'Oiva, Oivi', 'Pasi', 'Helka, Helga'])
# June
namedays.append(['Teemu, Nikodemus', 'Venla', 'Orvokki', 'Toivo', 'Sulevi',
                 'Kustaa, Kyösti, Kustavi', 'Suvi', 'Salomo, Salomon',
                 'Ensio', 'Seppo', 'Impi, Immi', 'Esko', 'Raili, Raila',
                 'Kielo', 'Vieno, Viena', 'Päivi, Päivikki, Päivä', 'Urho',
                 'Tapio', 'Siiri', 'Into', 'Ahti, Ahto',
                 'Paula, Liina, Pauliina', 'Aatto, Aatu, Aadolf',
                 'Johannes, Juhani, Juha, Juho, Jukka, Jussi, Janne, Jani,' +
                 ' Juhana', 'Uuno',
                 'Jorma, Jarmo, Jarkko, Jarno, Jere, Jeremias',
                 'Elviira, Elvi', 'Leo',
                 'Pietari, Pekka, Petri, Petra, Petteri, Pekko',
                 'Päiviö, Päivö'])
# July
namedays.append(['Aaro, Aaron', 'Maria, Mari, Maija, Meeri, Maaria, Marika,' +
                 ' Maiju, Maikki, Kukka-Maaria', 'Arvo', 'Ulla, Ulpu',
                 'Unto, Untamo', 'Esa, Esaias', 'Klaus, Launo',
                 'Turo, Turkka', 'Ilta, Jasmin', 'Saima, Saimi',
                 'Elli, Noora, Nelli, Eleonoora', 'Hermanni, Herkko, Herman',
                 'Ilari, Lari, Joel', 'Aliisa', 'Rauni, Rauna', 'Reino',
                 'Ossi, Ossian', 'Riikka', 'Saara, Sari, Salli, Salla, Sara',
                 'Marketta, Maarit, Reeta, Maaret, Reetta, Margareeta',
                 'Johanna, Hanna, Jenni, Jenna, Hannele, Hanne, Jonna',
                 'Leena, Leeni, Lenita, Mataleena', 'Oili, Olga',
                 'Kirsti, Tiina, Kirsi, Kristiina, Krista',
                 'Jaakko, Jaakoppi, Jaakob', 'Martta', 'Heidi', 'Atso',
                 'Olavi, Olli, Uolevi, Uoti', 'Asta', 'Helena, Elena'])
# August
namedays.append(['Maire', 'Kimmo', 'Linnea, Nea, Vanamo', 'Veera',
                 'Salme, Sanelma', 'Toimi, Keimo', 'Lahja',
                 'Sylvi, Sylvia, Silva', 'Erja, Eira', 'Lauri, Lasse, Lassi',
                 'Sanna, Susanna, Sanni', 'Klaara', 'Jesse',
                 'Onerva, Kanerva',
                 'Marjatta, Marja, Jaana, Marjo, Marjut, Marjaana, Marjukka' +
                 ', Marita, Maritta, Marianne, Marianna', 'Aulis', 'Verneri',
                 'Leevi', 'Mauno, Maunu', 'Samuli, Sami, Samuel, Samu',
                 'Soini, Veini', 'Iivari, Iivo', 'Varma, Signe', 'Perttu',
                 'Loviisa', 'Ilma, Ilmi, Ilmatar', 'Rauli', 'Tauno',
                 'Iines, Iina, Inari', 'Eemil, Eemeli', 'Arvi'])
# September
namedays.append(['Pirkka', 'Sinikka, Sini', 'Soili, Soile, Soila', 'Ansa',
                 'Mainio', 'Asko', 'Arho, Arhippa', 'Taimi', 'Eevert, Isto',
                 'Kalevi, Kaleva', 'Santeri, Ali, Ale, Aleksanteri',
                 'Valma, Vilja', 'Orvo', 'Iida', 'Sirpa',
                 'Hellevi, Hillevi, Hille, Hilla', 'Aili, Aila',
                 'Tyyne, Tytti, Tyyni', 'Reija', 'Varpu, Vaula', 'Mervi',
                 'Mauri', 'Mielikki', 'Alvar, Auno', 'Kullervo', 'Kuisma',
                 'Vesa', 'Arja', 'Mikko, Mika, Mikael, Miika, Miikka',
                 'Sorja, Sirja'])
# October
namedays.append(['Rauno, Rainer, Raine, Raino', 'Valio', 'Raimo',
                 'Saila, Saija', 'Inkeri, Inka', 'Minttu, Pinja',
                 'Pirkko, Pirjo, Piritta, Pirita, Birgitta', 'Hilja',
                 'Ilona', 'Aleksi, Aleksis', 'Otso, Ohto', 'Aarre, Aarto',
                 'Taina, Tanja, Taija', 'Elsa, Else, Elsi', 'Helvi, Heta',
                 'Sirkka, Sirkku', 'Saini, Saana', 'Satu, Säde', 'Uljas',
                 'Kauno, Kasperi', 'Ursula', 'Anja, Anita, Anniina, Anitta',
                 'Severi', 'Asmo', 'Sointu', 'Amanda, Niina, Manta',
                 'Helli, Hellä, Hellin, Helle', 'Simo', 'Alfred, Urmas',
                 'Eila', 'Artturi, Arto, Arttu'])
# November
namedays.append(['Pyry, Lyly', 'Topi, Topias', 'Terho', 'Hertta', 'Reima',
                 'Kustaa Aadolf', 'Taisto', 'Aatos', 'Teuvo', 'Martti',
                 'Panu', 'Virpi', 'Ano, Kristian', 'Iiris',
                 'Janika, Janita, Janina', 'Aarne, Aarno, Aarni',
                 'Eino, Einar, Einari', 'Tenho, Jousia',
                 'Liisa, Eliisa, Elisa, Elisabet, Liisi', 'Jalmari, Jari',
                 'Hilma', 'Silja, Selja', 'Ismo', 'Lempi, Lemmikki, Sivi',
                 'Katri, Kaisa, Kaija, Katja, Kaarina, Katariina, Kati' +
                 ', Kaisu, Riina', 'Sisko', 'Hilkka', 'Heini', 'Aimo',
                 'Antti, Antero, Atte'])
# December
namedays.append(['Oskari', 'Anelma, Unelma', 'Vellamo, Meri', 'Airi, Aira',
                 'Selma', 'Niilo, Niko, Niklas', 'Sampsa', 'Kyllikki, Kylli',
                 'Anna, Anne, Anni, Anu, Annikki, Anneli, Annukka, Annika',
                 'Jutta', 'Taneli, Tatu, Daniel', 'Tuovi', 'Seija', 'Jouko',
                 'Heimo', 'Auli, Aulikki', 'Raakel', 'Aapo, Aappo, Rami',
                 'Iikka, Iiro, Iisakki, Isko', 'Benjamin, Kerkko',
                 'Tuomas, Tuomo, Tommi, Tomi', 'Raafael', 'Senni',
                 'Aatami, Eeva, Eevi, Eveliina', '-', 'Tapani, Teppo, Tahvo',
                 'Hannu, Hannes', 'Piia', 'Rauha', 'Daavid, Taavetti, Taavi',
                 'Sylvester, Silvo'])

def wday_str(i):
    i = i % 7
    return ['su', 'ma', 'ti', 'ke', 'to', 'pe', 'la'][i]

def get_nameday(month, day):
    return namedays[month - 1][day - 1]

def next_namedays(n):
    """Return list of nameday structs for n days

    Each struct has:
       - month
       - day
       - weekday abbreviation of two letters in Finnish
       - names for that date

    """
    
    secs = [86400 * x for x in range(n)]

    dates = [time.localtime(time.time() + x) for x in secs]

    month = [int(time.strftime('%m', x)) for x in dates]
    day = [int(time.strftime('%d', x)) for x in dates]
    wday = [int(time.strftime('%w', x)) for x in dates]

    names = [[] for x in range(n)]
    for i in range(n):
        names[i].append(month[i])
        names[i].append(day[i])
        names[i].append(wday_str(wday[i]))
        names[i].append(get_nameday(month[i], day[i]))
        
    return names

if __name__ == '__main__':

    # Output:
    # su: Valma, Vilja
    # ma: Orvo
    # ti: Iida

    for row in next_namedays(3):
        print '%s: %s' % (row[2], row[3])  # weekday and names



"""
Warns about large files

$Id: module_urlsize.py 331 2012-11-13 10:18:18Z riku.lindblad@gmail.com $
$HeadURL: https://pyfibot.googlecode.com/svn/trunk/pyfibot/modules/module_urlsize.py $
"""


def handle_url(bot, user, channel, url, msg):
    """inform about large files (over 5MB)"""

    # TODO: Hard-coded
    if channel == "#wow":
        return
    size = getUrl(url).getSize()
    headers = getUrl(url).getHeaders()
    if 'content-type' in headers:
        contentType = headers['content-type']
    else:
        contentType = "Unknown"
    if not size:
        return
    size = size / 1024
    if size > 5:
        return bot.say(channel, "File size: %s MB - Content-Type: %s" % (size, contentType))



#!/usr/bin/python
# -*- coding: utf-8 -*-
## (c)2004 Timo Reunanen <parker _et_ wolfenstein _dit_ org>

## Thanks to Shrike for helping this :)

## Made by #python.fi ppl for python ppl

"""Find absolute URLs from strings"""

__version__ = "3.0.3"

import re
from UserString import UserString
import string
import sys

_countrycodes=['ac', 'ad', 'ae', 'af', 'ag', 'ai', 'al', 'am', 'an', 'ao',
               'aq', 'ar', 'as', 'at', 'au', 'aw', 'ax', 'az', 'ba', 'bb',
               'bd', 'be', 'bf', 'bg', 'bh', 'bi', 'bj', 'bm', 'bn', 'bo',
               'br', 'bs', 'bt', 'bv', 'bw', 'by', 'bz', 'ca', 'cc', 'cd',
               'cf', 'cg', 'ch', 'ci', 'ck', 'cl', 'cm', 'cn', 'co', 'cr',
               'cs', 'cu', 'cv', 'cx', 'cy', 'cz', 'de', 'dj', 'dk', 'dm',
               'do', 'dz', 'ec', 'ee', 'eg', 'eh', 'er', 'es', 'et', 'fi',
               'fj', 'fk', 'fm', 'fo', 'fr', 'ga', 'gb', 'gd', 'ge', 'gf',
               'gg', 'gh', 'gi', 'gl', 'gm', 'gn', 'gp', 'gq', 'gr', 'gs',
               'gt', 'gu', 'gw', 'gy', 'hk', 'hm', 'hn', 'hr', 'ht', 'hu',
               'id', 'ie', 'il', 'im', 'in', 'io', 'iq', 'ir', 'is', 'it',
               'je', 'jm', 'jo', 'jp', 'ke', 'kg', 'kh', 'ki', 'km', 'kn',
               'kp', 'kr', 'kw', 'ky', 'kz', 'la', 'lb', 'lc', 'li', 'lk',
               'lr', 'ls', 'lt', 'lu', 'lv', 'ly', 'ma', 'mc', 'md', 'mg',
               'mh', 'mk', 'ml', 'mm', 'mn', 'mo', 'mp', 'mq', 'mr', 'ms',
               'mt', 'mu', 'mv', 'mw', 'mx', 'my', 'mz', 'na', 'nc', 'ne',
               'nf', 'ng', 'ni', 'nl', 'no', 'np', 'nr', 'nu', 'nz', 'om',
               'pa', 'pe', 'pf', 'pg', 'ph', 'pk', 'pl', 'pm', 'pn', 'pr',
               'ps', 'pt', 'pw', 'py', 'qa', 're', 'ro', 'ru', 'rw', 'sa',
               'sb', 'sc', 'sd', 'se', 'sg', 'sh', 'si', 'sj', 'sk', 'sl',
               'sm', 'sn', 'so', 'sr', 'st', 'sv', 'sy', 'sz', 'tc', 'td',
               'tf', 'tg', 'th', 'tj', 'tk', 'tl', 'tm', 'tn', 'to', 'tp',
               'tr', 'tt', 'tv', 'tw', 'tz', 'ua', 'ug', 'uk', 'um', 'us',
               'uy', 'uz', 'va', 'vc', 've', 'vg', 'vi', 'vn', 'vu', 'wf',
               'ws', 'ye', 'yt', 'yu', 'za', 'zm', 'zw', 'aero', 'biz', 'com',
               'coop', 'edu', 'gov', 'info', 'int', 'mil', 'museum', 'name',
               'net', 'org', 'pro', 'me', 'mobi', 'xxx']

_schemes = ['http://', 'https://', 'ftp://', 'svn://']

## domainlabel   = alphanum | alphanum *( alphanum | "-" ) alphanum
_domainlabel=r'''
(?:
  [a-z0-9]
  |
  (?:
    [a-z0-9]
    [a-z0-9-]*?
    [a-z0-9]
  )
)
'''

## toplabel      = alpha | alpha *( alphanum | "-" ) alphanum
_toplabel=r'''
(?P<toplabel>
  (?:
    [a-z]
    [a-z0-9-]*
    [a-z0-9]
  )
  |
  (?: [a-z] )
)
'''


## hostname      = *( domainlabel "." ) toplabel [ "." ]
_hostname=r'''
(?P<hostname>
  (?: %(_domainlabel)s [.])+
  %(_toplabel)s
)
''' % globals()

_ipv4=r"""
(?P<ipv4>
  (?:(?<=[^0-9]) | \A)                  ## Doesn't start with number
  (?:[0-9]{1,3}\.){3}[0-9]{1,3}         ## '123.123.123.123'
  (?![0-9])                             ## And doesn't end with number
)
"""

_singleNum=r"""[0-9a-f]{0,4}"""
_ipv6=r"""
(?:
  [[]                                    ## Start [
  (?P<ipv6>
    (?: %(_singleNum)s[:] ){2,7}         ## xxxx:xxxx:[xxxx:[xxxx:[xxxx:[xxxx:[xxxx:]]]]]
    %(_singleNum)s                       ## xxxx
  )
  []]                                    ## End ]
)
""" % globals()

## host          = hostname | IPv4address | IPv6address [ ":" port ]
_host=r'''
(?P<host>
  (?:                                   ## Must be one of these
    %(_hostname)s |                     
    %(_ipv4)s |
    %(_ipv6)s
  )
  (?:                                   ## Check if port is defined
    :
     (?P<port>
      [0-9]+
    )
  )?
)
''' % globals()

## def emacsHack():
##   pass
## "

validPathChars=[string.ascii_letters,
                string.digits,
                '/;?:@&=+$,%#'
                '-_.!~*()',
                'åäö[]<>{}^\|\'`', ## not so valid but used
                ]

validUserinfoChars=[string.ascii_letters,
                    string.digits,
                    ';:&=+$,',
                    '-_.!~*()',
                    '%@'
                    ]

validPathChars=''.join(validPathChars)
validUserinfoChars=''.join(validUserinfoChars)

hostRe=re.compile(_host, re.VERBOSE | re.MULTILINE | re.I)

def grab(txt, needScheme=True):
    seekpos=0

    possibleUrls=[]

    ## Try to find all possible URI's 
    while 1:
        srch=hostRe.search(txt, seekpos)
        if not srch: break

        valid=False

        ## Find possible URI with valid hostname, ipv4 or ipv6
        if srch.group('host'):
            if srch.group('ipv4'):
                ## Check if it is valid ipv4
                ip=[x for x in srch.group('ipv4').split('.') if int(x) > 255]
                if ip == []:
                    valid=True
                    
            elif srch.group('hostname'):
                ## Check if valid country code
                if srch.group('toplabel') in _countrycodes:
                    valid=True

            elif srch.group('ipv6'):
                ## ipv6 is always valid :P (yeah right...) 
                valid=True

        if not valid:
            seekpos=srch.end('host')
            continue
        
        s=srch.start('host')
        e=srch.end('host')

        ## Check if there is userinfo in URI
        if s > 1 and txt[s-1]=='@':
            s-=1

            while s >= 0 and txt[s] in validUserinfoChars: s-=1

            s+=1

        ## Check if there is scheme
        schemeFound=False
        for scheme in _schemes:
            if txt[s-len(scheme):s] == scheme:
                s-=len(scheme)
                schemeFound=True
                break

        ## If scheme neede and no scheme found, skip it
        if needScheme and not schemeFound:
            seekpos=srch.end('host')
            continue

        ## Try to figure out rest of URI
        while e < len(txt) and txt[e] in validPathChars: e+=1


        ## Some silly hack
        ns=s

        for v in [('<', '>'),('(', ')'), ('{', '}'), ('[', ']'), '"', "'"]:
            if type(v)==tuple:
                sc,ec=v
            else:
                sc=ec=v

            if txt[ns-len(sc):].startswith(sc):
                if txt[e-len(ec):].startswith(ec):
                    e-=len(ec)

        possibleUrls.append(txt[s:e])

        seekpos=e

    return possibleUrls

def unitTest():
    import unittest
    class TestPRUL(unittest.TestCase):
        def setUp(self):
            self.needScheme = True

        def testLeadingSpaces(self):
            """Leading spaces before URL"""
            self.assertEqual(["http://tomtom.foobar.org/"], grab('     http://tomtom.foobar.org/', self.needScheme))
            self.assertEqual(["http://www.foobi.org/saatoimia"], grab('  http://www.foobi.org/saatoimia', self.needScheme))

        def testTrailingSpaces(self):
            """Trailing spaces after URL"""
            self.assertEqual(["http://tomtom.foobar.org/"], grab('http://tomtom.foobar.org/     ', self.needScheme))
            self.assertEqual(["http://www.foobi.org/saatoimia"], grab('http://www.foobi.org/saatoimia    ', self.needScheme))

        def testLongURL(self):
            """A long URL-like string"""
            self.assertEqual([], grab('www.www.www.www.www.www.www.www.www.www.www.www.www.www.www.www.www.www.www.www.www.www.www.www.www.www.www.www.www.www.www.www.www.www.www.www.www.www.www.www.www.www.www.www.www.www.www.www.www.www.www.www.www', self.needScheme))

        def testQuestionMarkURI(self):
            """An URI with a question mark"""
            self.assertEqual(["http://www.bdog.fi/cgi-bin/netstore/tuotehaku.pl?tuoteryhma=16"],  grab('http://www.bdog.fi/cgi-bin/netstore/tuotehaku.pl?tuoteryhma=16', self.needScheme))

        def testLeadingText(self):
            """Leading text"""
            self.assertEqual(["http://www.technikfoo.de/mai"], grab('here it is: http://www.technikfoo.de/mai', self.needScheme))

        def testLeadingAndTrailingText(self):
            """Leading and trailing text"""
            self.assertEqual(["http://123.123.123.123"], grab('fooasdf asdf a http://123.123.123.123 asdfasdf', self.needScheme))

        def testIP(self):
            """http URI with an ip number"""
            self.assertEqual(["http://234.234.234.234"], grab('http://234.234.234.234', self.needScheme))

        def testFoobarIP(self):
            """ip number-like text"""
            self.assertEqual([], grab('http://11123.123.123.123/eisaa http://123.123.123.12345/eisaa', self.needScheme))

        def test2URIs(self):
            """2 URIs on same text"""
            self.assertEqual(["http://foobar.fi/1234{}[]{}", "http://127.0.0.1/"], grab('http://foobar.fi/1234{}[]{} sadfljs dlfkjsd lf;asdf http://127.0.0.1/', self.needScheme))
            
        def testIPv6(self):
            """IPv6 URL with scheme"""
            self.assertEqual(["http://[2001:a68:104:1337:250:daff:fe72:871c]/toimia"], grab('foo http://[2001:a68:104:1337:250:daff:fe72:871c]/toimia', self.needScheme))

        def testIPv6noscheme(self):
            """IPv6 URL without a scheme"""
            if self.needScheme: return
            
            self.assertEqual(["[2001:a68:104:1337:250:daff:fe72:871c]/toimia"], grab('foo [2001:a68:104:1337:250:daff:fe72:871c]/toimia', self.needScheme))

        def testNoScheme(self):
            """URI without a scheme"""
            if self.needScheme: return
            
            self.assertEqual(["123.123.123.123"], grab('123.123.123.123', self.needScheme))

        def testRedirect(self):
            """Redirect URL"""
            self.assertEqual(['http://rediretinmyurl.com/http://dest.url.org/1/2/3/4?434', 'http://secondurl.com', 'ftp://1.2.3.4/adsfasdf'], grab('http://rediretinmyurl.com/http://dest.url.org/1/2/3/4?434 http://secondurl.com ftp://1.2.3.4/adsfasdf', self.needScheme))

        def testAnchor(self):
            """Link with an anchor tag"""
            self.assertEqual(['http://foo.com/page.html#anchor'], grab('http://foo.com/page.html#anchor', self.needScheme))

        def testScandinavian(self):
            """Test åäö"""
            self.assertEqual(['http://www.hs.fi/kotimaa/artikkeli/Äidin+avovaimosta+lapsen+toinen+huoltaja+KKOn+päätöksellä/1135253379084'], grab('http://www.hs.fi/kotimaa/artikkeli/Äidin+avovaimosta+lapsen+toinen+huoltaja+KKOn+päätöksellä/1135253379084', self.needScheme))

        def testBlocks(self):
            """Test blocks"""
            self.assertEqual(['http://link1.com',
                              'http://link2.com',
                              'http://link3.com',
                              'http://link4.com',
                              'http://link5.com',
                              ], grab('(http://link1.com) <http://link2.com> "http://link3.com" \'http://link4.com\' [http://link5.com]', self.needScheme))

    # run unittest
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestPRUL))
    unittest.TextTestRunner(verbosity=2).run(suite)

def _main():
    if '--test' in sys.argv:
        unitTest()
        return

    needScheme='--noscheme' not in sys.argv
    
    for line in sys.stdin:
        for url in grab(line, needScheme):
            print url

if __name__ == '__main__':
    _main()



"""Upload images to imgur.com

$Id: module_imgur.py 331 2012-11-13 10:18:18Z riku.lindblad@gmail.com $
$HeadURL: https://pyfibot.googlecode.com/svn/trunk/pyfibot/modules/available/module_imgur.py $
"""

import urllib
try:
    import json
    import oauth2 as oauth
    init_ok = True
except:
    init_ok = False

consumer_key = 'cbcbcaf338bc7b430428dc4b64500e7004c761bbd'
consumer_secret = '1e4f5bdff44ce1b2a8ec069d8a293991'
oauth_token = 'a0b19c8a764017234fedd83c095911d804c762172'
oauth_token_secret = '9b2033e1d48be6aca606bacba2901300'
consumer = None
token = None


def init(bot):
    if not init_ok:
        return False
    global config
    global consumer_key, consumer_secret, oauth_token, oauth_token_secret
    global token, consumer
    try:
        config = config = bot.config.get("module_imgur", {})
    except KeyError:
        config = None

    consumer_key = config.get('consumer_key', '')
    consumer_secret = config.get('consumer_secret', '')
    oauth_token = config.get('oauth_token', '')
    oauth_token_secret = config.get('oauth_token_secret', '')

    token = oauth.Token(key=oauth_token, secret=oauth_token_secret)
    consumer = oauth.Consumer(key=consumer_key, secret=consumer_secret)


def handle_url(bot, user, channel, url, msg):
    if init_ok and (url.endswith(".jpg") or url.endswith(".gif")):
        print channel, upload_images([url])


def upload_gallery(url):
    from mechanize import Browser
    br = Browser()
    br.set_handle_robots(False)
    br.open(url)

    urls = set()

    for link in br.links(url_regex=".jpg$"):
        urls.add(link.url)

    upload_images(urls)


def upload_images(urls):
    # uploaded images
    images = []

    client = oauth.Client(consumer, token)

    # Transload image(s) to imgur
    for url in urls:
        print "Transloading %s..." % url

        metadata = {
            'image': url,
            'type': 'url',
            'caption': 'Original url: %s' % url}

        resp, cont = client.request("http://api.imgur.com/2/account/images.json", "POST", body=urllib.urlencode(metadata))
        data = json.loads(cont)
        if resp.status != 200:
            print resp
            errmsg = data['error']['message']
            print "Transload error for %s: %s " % (url, errmsg)
            if errmsg == "API limits exceeded":
                print "API limits exceeded, aborting uploads"
                return
            else:
                continue

        images.append(data['images']['image']['hash'])

    # check that the bot album exists
    resp, cont = client.request("http://api.imgur.com/2/account/albums.json", "GET")
    data = json.loads(cont)
    album = [album for album in data['albums'] if album['title'] == "Pyfibot"]
    if len(album) != 1:
        print "Album not found, please create it"
        return

    album = album[0]
    aid = album['id']
    link = album['anonymous_link']

    # add images to bot album
    resp, cont = client.request("http://api.imgur.com/2/account/albums/%s.json" % aid, "POST", body=urllib.urlencode({'add_images': ",".join(images)}))

    return "Images (%s) uploaded to %s (%s)" % (",".join(images), link, aid)



"""
$Id: module_pgsqlseries.py 331 2012-11-13 10:18:18Z riku.lindblad@gmail.com $
$HeadURL: https://pyfibot.googlecode.com/svn/trunk/pyfibot/modules/available/module_pgsqlseries.py $
"""

import time
import datetime
try:
    from pyPgSQL import PgSQL
except:
    pass


def init(bot):
    global config
    try:
        config = bot.config["module_pgsqlseries"]
    except KeyError:
        config = None


def command_ep(bot, user, channel, args):
    """Usage: sqlep [today|yesterday|tomorrow] or [seriename]"""
    if not config:
        return
    cx = PgSQL.connect(database=config["database"],
                       host=config["host"],
                       user=config["user"],
                       password=config["password"])
    cur = cx.cursor()

    if not args:
        bot.say(channel, "Usage: ep [today|yesterday|tomorrow] or [name of series]")
        return

    # handle special arguments
    if args in ["today", "tomorrow", "yesterday"]:
        cur.execute("SELECT * FROM series.episodes WHERE airdate = '%s';" % args)
        cur.fetchone()
        if cur.rowcount in [-1, 0]:
            bot.say(channel, "No known releases %s" % args)
            return
    else:
        # try to find the serie
        cur.execute("SELECT * FROM series.episodes WHERE series ~* %s AND airdate >= 'today' ORDER BY airdate LIMIT 1", (args,))
        cur.fetchone()
        # not found
        if cur.rowcount in [-1, 0]:
            # do we know anything like it?
            cur.execute("SELECT * FROM series.episodes WHERE series ~* %s", (args,))
            res = cur.fetchone()
            # no idea what you're trying to say, bub
            if cur.rowcount in [-1, 0]:
                bot.say(channel, "Series '%s' not found" % args)
                return
            # known serie, just no eps stored
            else:
                bot.say(channel, "No unaired episodes of '%s' found" % res['series'])
                return

    # rewind the cursor to counter the fetchone-hacks
    cur.rewind()
    episodes = []
    # go through the results
    for row in cur.fetchall():
        serie = row['series']
        season = row['season']
        episode = row['episode']
        title = row['title']
        airdate = row['airdate']
        if episode < 10:
            episode = "0%d" % episode  # pad ep with zeroes
        ad = datetime.date.fromtimestamp(airdate.ticks())
        now = datetime.date.today()
        tomorrow = now + datetime.timedelta(days=1)
        td = ad - now
        # change 0 and 1 to today & tomorrow, don't show date if we're asking stuff for a certain day
        airdatestr = ""
        airdate = ad  # use datetime instead of the postgres internal type
        if td.days >= 0:
            if ad == now:
                if args != "today":
                    airdatestr = "on %s (Today)" % airdate
            elif ad == tomorrow:
                if args != "tomorrow":
                    airdatestr = "on %s (Tomorrow)" % airdate
            else:
                airdatestr = "on %s (%d days)" % (airdate, td.days)

        episodes.append("%s %sx%s '%s' %s" % (serie, season, episode, title, airdatestr))
    bot.say(channel, "-- ".join(episodes))
    if not cur.closed:
        cur.close()
    cx.close()



# -*- coding: utf-8 -*-

"""
Bot core

@author Riku 'Shrike' Lindblad (shrike@addiktit.net)
@copyright Copyright (c) 2004 Riku Lindblad
@license New-Style BSD

$Id: botcore.py 331 2012-11-13 10:18:18Z riku.lindblad@gmail.com $
$HeadURL: https://pyfibot.googlecode.com/svn/trunk/pyfibot/botcore.py $
"""

# twisted imports
from twisted.words.protocols import irc
from twisted.internet import reactor, threads
from twisted.python import rebuild

from types import FunctionType

import string
import logging
from util import pyfiurl

# line splitting
import textwrap

__pychecker__ = 'unusednames=i, classattr'

log = logging.getLogger("bot")


class CoreCommands(object):
    def command_echo(self, user, channel, args):
        self.say(channel, "%s: %s" % (user, args))

    def command_ping(self, user, channel, args):
        self.say(channel, "%s: My current ping is %.0fms" %
                 (self.factory.getNick(user), self.pingAve * 100.0))

    def command_rehash(self, user, channel, args):
        """Reload modules. Usage: rehash [debug]"""

        if self.factory.isAdmin(user):
            try:
                # rebuild core & update
                log.info("rebuilding %r" % self)
                rebuild.updateInstance(self)

                self.factory._loadmodules()

            except Exception, e:
                self.say(channel, "Rehash error: %s" % e)
                log.error("Rehash error: %s" % e)
            else:
                self.say(channel, "Rehash OK")
                log.info("Rehash OK")

    def say(self, channel, message, length=None):
        """Must be implemented by the inheriting class"""
        raise NotImplementedError

    def command_join(self, user, channel, args):
        """Usage: join <channel>[@network] [password] - Join the specified channel"""

        if not self.factory.isAdmin(user):
            return

        password = None
        # see if we have multiple arguments
        try:
            args, password = args.split(' ', 1)
        except ValueError:
            pass

        # see if the user specified a network
        try:
            newchannel, network = args.split('@', 1)
        except ValueError:
            newchannel, network = args, self.network.alias
        try:
            bot = self.factory.allBots[network]
        except KeyError:
            self.say(channel, "I am not on that network.")
        else:
            log.debug("Attempting to join channel %s", channel)
            if newchannel in bot.network.channels:
                self.say(channel, "I am already in %s on %s." % (newchannel, network))
                log.debug("Already on channel %s" % channel)
                log.debug("Channels I'm on this network: %s" % bot.network.channels)
            else:
                if password:
                    bot.join(newchannel, key=password)
                    log.debug("Joined")
                else:
                    bot.join(newchannel)
                    log.debug("Joined")

    # alias of part
    def command_leave(self, user, channel, args):
        """Usage: leave <channel>[@network] - Leave the specified channel"""
        self.command_part(user, channel, args)

    def command_part(self, user, channel, args):
        """Usage: part <channel>[@network] - Leave the specified channel"""

        if not self.factory.isAdmin(user):
            return

        # part what and where?
        try:
            newchannel, network = args.split('@', 1)
        except ValueError:
            newchannel, network = args, self.network.alias

        # get the bot instance for this chat network
        try:
            bot = self.factory.allBots[network]
        except KeyError:
            self.say(channel, "I am not on that network.")
        else:
            if newchannel not in bot.network.channels:
                self.say(channel, "I am not in %s on %s." % (newchannel, network))
                self.say(channel, "I am on %s" % bot.network.channels)
            else:
                bot.network.channels.remove(newchannel)
                bot.part(newchannel)

    def command_quit(self, user, channel, args):
        """Usage: logoff - Leave this network"""

        if not self.factory.isAdmin(user):
            return

        self.quit("Working as programmed")
        self.hasQuit = 1

    def command_channels(self, user, channel, args):
        """Usage: channels <network> - List channels the bot is on"""
        if not args:
            self.say(channel, "Please specify a network")
            return

        self.say(channel, "I am on %s" % self.network.channels)

    def command_help(self, user, channel, cmnd):
        """Get help on all commands or a specific one. Usage: help [<command>]"""

        commands = []
        for module, env in self.factory.ns.items():
            myglobals, mylocals = env
            commands += [(c.replace("command_", ""), ref) for c, ref in mylocals.items() if c.startswith("command_%s" % cmnd)]
        # Help for a specific command
        if len(cmnd) > 0:
            for cname, ref in commands:
                if cname == cmnd:
                    helptext = ref.__doc__.split("\n", 1)[0]
                    self.say(channel, "Help for %s: %s" % (cmnd, helptext))
                    return
        # Generic help
        else:
            commandlist = ", ".join([c for c, ref in commands])
            self.say(channel, "Available commands: %s" % commandlist)


class PyFiBot(irc.IRCClient, CoreCommands):
    """PyFiBot"""

    nickname = "pyfibot"
    realname = "http://code.google.com/p/pyfibot/"
    password = None

    # send 1 msg per second max
    linerate = 1
    hasQuit = False

    CMDCHAR = "."

    # Rolling ping time average
    pingAve = 0.0

    def __init__(self, network):
        self.network = network
        self.nickname = self.network.nickname
        self.linerate = self.network.linerate
        # Text wrapper to clip overly long answers
        self.tw = textwrap.TextWrapper(width=400, break_long_words=True)
        log.info("bot initialized")

    def __repr__(self):
        return 'PyFiBot(%r, %r)' % (self.nickname, self.network.address)

    """Core"""
    def printResult(self, msg, info):
        # Don't print results if there is nothing to say (usually non-operation on module)
        if msg:
            log.debug("Result %s %s" % (msg, info))

    def printError(self, msg, info):
        log.error("ERROR %s %s" % (msg, info))

    def connectionMade(self):
        irc.IRCClient.connectionMade(self)
        self.repeatingPing(300)
        log.info("connection made")

    def connectionLost(self, reason):
        irc.IRCClient.connectionLost(self, reason)
        log.info("connection lost: %s", reason)

    def signedOn(self):
        """Called when bot has succesfully signed on to server."""
        # QuakeNet specific auth and IP-address masking
        # TODO: Alias could be case-sensitive
        if self.network.alias == "quakenet":
            log.info("I'm on quakenet, authenticating...")
            self.mode(self.nickname, '+', 'x')  # Hide ident
            authname = self.factory.config['networks']['quakenet'].get('authname', None)
            authpass = self.factory.config['networks']['quakenet'].get('authpass', None)
            if not authname or not authpass:
                log.info("Quakenet authname or authpass not found. Authentication aborted")
            else:
                self.say("Q@CServe.quakenet.org", "AUTH %s %s" % (authname, authpass))
                log.info("Auth sent.")
        for chan in self.network.channels:
            # Defined as a tuple, channel has a key
            if type(chan) == list:
                self.join(chan[0], key=chan[1])
            else:
                self.join(chan)

        log.info("joined %d channel(s): %s" % (len(self.network.channels), ", ".join(self.network.channels)))
        self._runEvents("signedon")

    def pong(self, user, secs):
        self.pingAve = ((self.pingAve * 5) + secs) / 6.0

    def repeatingPing(self, delay):
        reactor.callLater(delay, self.repeatingPing, delay)
        self.ping(self.nickname)

    def say(self, channel, message, length=None):
        """Override default say to make replying to private messages easier"""
        # wrap long text into suitable fragments
        msg = self.tw.wrap(message)
        cont = False

        for m in msg:
            if cont:
                m = "..." + m
            self.msg(channel, m, length)
            cont = True
        return ('botcore.say', channel, message)

    def log(self, message):
        botId = "%s@%s" % (self.nickname, self.network.alias)
        log.info("%s: %s", botId, message)

    def callLater(self, delay, callable):
        self.callLater(delay, callable)

    """Communication"""
    def privmsg(self, user, channel, msg):
        """This will get called when the bot receives a message.
        @param user: nick!user@host
        @param channel: Channel where the message originated from
        @param msg: The actual message
        """

        channel = channel.lower()
        lmsg = msg.lower()
        lnick = self.nickname.lower()
        nickl = len(lnick)
        if channel == lnick:
            # Turn private queries into a format we can understand
            if not msg.startswith(self.CMDCHAR):
                msg = self.CMDCHAR + msg
            elif lmsg.startswith(lnick):
                msg = self.CMDCHAR + msg[nickl:].lstrip()
            elif lmsg.startswith(lnick) and len(lmsg) > nickl and lmsg[nickl] in string.punctuation:
                msg = self.CMDCHAR + msg[nickl + 1:].lstrip()
        else:
            # Turn 'nick:' prefixes into self.CMDCHAR prefixes
            if lmsg.startswith(lnick) and len(lmsg) > nickl and lmsg[nickl] in string.punctuation:
                msg = self.CMDCHAR + msg[len(self.nickname) + 1:].lstrip()
        reply = (channel == lnick) and user or channel

        if msg.startswith(self.CMDCHAR):
            cmnd = msg[len(self.CMDCHAR):]
            self._command(user, reply, cmnd)

        # Run privmsg handlers
        self._runhandler("privmsg", user, reply, msg)

        # run URL handlers
        urls = pyfiurl.grab(msg)
        if urls:
            for url in urls:
                self._runhandler("url", user, reply, url, msg)

    def _runhandler(self, handler, *args, **kwargs):
        """Run a handler for an event"""
        handler = "handle_%s" % handler
        # module commands
        for module, env in self.factory.ns.items():
            myglobals, mylocals = env
            # find all matching command functions
            handlers = [(h, ref) for h, ref in mylocals.items() if h == handler and type(ref) == FunctionType]

            for hname, func in handlers:
                # defer each handler to a separate thread, assign callbacks to see when they end
                # TODO: Profiling: add time.time() to callback params, calculate difference
                d = threads.deferToThread(func, self, *args, **kwargs)
                d.addCallback(self.printResult, "handler %s completed" % hname)
                d.addErrback(self.printError, "handler %s error" % hname)

    def _runEvents(self, eventname, *args, **kwargs):
        """Run funtions on events named by eventname parameter"""
        eventname = "event_%s" % eventname
        for module, env in self.factory.ns.items():
            myglobals, mylocals = env

            # find all matching events
            events = [(h, ref) for h, ref in mylocals.items() if h == eventname and type(ref) == FunctionType]

            for ename, func in events:
                # defer each handler to a separate thread, assign callbacks to see when they end
                # TODO: Profiling: add time.time() to callback params, calculate difference
                d = threads.deferToThread(func, self, *args, **kwargs)
                d.addCallback(self.printResult, "%s %s event completed" % (module, ename))
                d.addErrback(self.printError, "%s %s event error" % (module, ename))

    def _command(self, user, channel, cmnd):
        """Handles bot commands.

        This function calls the appropriate method for the given command.

        The command methods are formatted as "command_<commandname>"
        """
        # Split arguments from the command part
        try:
            cmnd, args = cmnd.split(" ", 1)
        except ValueError:
            args = ""

        # core commands
        method = getattr(self, "command_%s" % cmnd, None)
        if method is not None:
            log.info("internal command %s called by %s (%s) on %s" % (cmnd, user, self.factory.isAdmin(user), channel))
            method(user, channel, args)
            return

        # module commands
        for module, env in self.factory.ns.items():
            myglobals, mylocals = env
            # find all matching command functions
            commands = [(c, ref) for c, ref in mylocals.items() if c == "command_%s" % cmnd]

            for cname, command in commands:
                log.info("module command %s called by %s (%s) on %s" % (cname, user, self.factory.isAdmin(user), channel))
                # Defer commands to threads
                d = threads.deferToThread(command, self, user, channel, args)
                d.addCallback(self.printResult, "command %s completed" % cname)
                d.addErrback(self.printError, "command %s error" % cname)

    ### LOW-LEVEL IRC HANDLERS ###

    def irc_JOIN(self, prefix, params):
        """override the twisted version to preserve full userhost info"""
        nick = self.factory.getNick(prefix)
        channel = params[-1]

        if nick == self.nickname:
            self.joined(channel)
        else:
            self.userJoined(prefix, channel)

        if nick.lower() != self.nickname.lower():
            pass
        elif channel not in self.network.channels:
            self.network.channels.append(channel)
            self.factory.setNetwork(self.network)

    def irc_PART(self, prefix, params):
        """override the twisted version to preserve full userhost info"""

        nick = self.factory.getNick(prefix)
        channel = params[0]

        if nick == self.nickname:
            self.left(channel)
        else:
            # some clients don't send a part message at all, compensate
            if len(params) == 1:
                params.append("")
            self.userLeft(prefix, channel, params[1])

    def irc_QUIT(self, prefix, params):
        """QUIT-handler.

        Twisted IRCClient doesn't handle this at all.."""

        nick = self.factory.getNick(prefix)
        if nick == self.nickname:
            self.left(params)
        else:
            self.userLeft(prefix, None, params[0])

    ###### HANDLERS ######

    ## ME
    def joined(self, channel):
        """I joined a channel"""
        self._runhandler("joined", channel)

    def left(self, channel):
        """I left a channel"""
        self._runhandler("left", channel)

    def noticed(self, user, channel, message):
        """I received a notice"""
        self._runhandler("noticed", user, channel, message)

    def modeChanged(self, user, channel, set, modes, args):
        """Mode changed on user or channel"""
        self._runhandler("modeChanged", user, channel, set, modes, args)

    def kickedFrom(self, channel, kicker, message):
        """I was kicked from a channel"""
        self._runhandler("kickedFrom", channel, kicker, message)

    def nickChanged(self, nick):
        """I changed my nick"""
        self._runhandler("nickChanged", nick)

    ## OTHER PEOPLE
    def userJoined(self, user, channel):
        """Someone joined"""
        self._runhandler("userJoined", user, channel)

    def userLeft(self, user, channel, message):
        """Someone left"""
        self._runhandler("userLeft", user, channel, message)

    def userKicked(self, kickee, channel, kicker, message):
        """Someone got kicked by someone"""
        self._runhandler("userKicked", kickee, channel, kicker, message)

    def action(self, user, channel, data):
        """An action"""
        self._runhandler("action", user, channel, data)

    def topicUpdated(self, user, channel, topic):
        """Save topic to maindb when it changes"""
        self._runhandler("topicUpdated", user, channel, topic)

    def userRenamed(self, oldnick, newnick):
        """Someone changed their nick"""
        self._runhandler("userRenamed", oldnick, newnick)

    def receivedMOTD(self, motd):
        """MOTD"""
        self._runhandler("receivedMOTD", motd)

    ## SERVER INFORMATION

    ## Network = Quakenet -> do Q auth
    def isupport(self, options):
        log.info(self.network.alias + " SUPPORTS: " + ",".join(options))

    def created(self, when):
        log.info(self.network.alias + " CREATED: " + when)

    def yourHost(self, info):
        log.info(self.network.alias + " YOURHOST: " + info)

    def myInfo(self, servername, version, umodes, cmodes):
        log.info(self.network.alias + " MYINFO: %s %s %s %s" % (servername, version, umodes, cmodes))

    def luserMe(self, info):
        log.info(self.network.alias + " LUSERME: " + info)



"""
$Id: module_webchat.py 331 2012-11-13 10:18:18Z riku.lindblad@gmail.com $
$HeadURL: https://pyfibot.googlecode.com/svn/trunk/pyfibot/modules/module_webchat.py $
"""

import socket


def handle_userJoined(bot, user, channel):
    nick = getNick(user)
    userhost = user.split("!")[1]
    username, host = userhost.split("@")
    username = username.replace("~", "").replace("-", "")
    # known webchat hosts
    if host in ["webchat.xs4all.nl", "wwwirc.kapsi.fi", "webchat.mibbit.com", "gateway/web/freenode"]:
        origin = webchat_getorigin(username)
        if origin:
            return bot.say(channel, "%s is using webchat from %s" % (nick, origin))


def command_webchat(bot, user, channel, args):
    """Parse a webhcat hex ip to a domain"""
    origin = webchat_getorigin(args)
    if origin:
        return bot.say(channel, "webchat from %s" % origin)
    else:
        return bot.say(channel, "%s: %s is not a valid webchat hex ip" % (getNick(user), args))


def webchat_getorigin(hexip):
    """Parse webchat hex-format ip to decimal ip and hostname if it exists"""
    if len(hexip) != 8:
        return
    ip = []
    for i in range(2, len(hexip) + 2, 2):
        try:
            dec = int(hexip[i - 2:i], 16)
        except ValueError:
            return
        ip.append(str(dec))
    if ip:
        origin = '.'.join(ip)
        hostname = socket.getfqdn(origin)
        if hostname != origin:
            origin = "%s -> %s" % (origin, hostname)
    return origin



"""
$Id: module_spotify.py 331 2012-11-13 10:18:18Z riku.lindblad@gmail.com $
$HeadURL: https://pyfibot.googlecode.com/svn/trunk/pyfibot/modules/module_spotify.py $
"""

import re


def handle_privmsg(bot, user, channel, args):
    """Grab Spotify URLs from the messages and handle them"""

    m = re.match(".*(http:\/\/open.spotify.com\/|spotify:)(album|artist|track)([:\/])([a-zA-Z0-9]+)\/?.*", args)
    if not m:
        return None

    apiurl = "http://ws.spotify.com/lookup/1/?uri=spotify:%s:%s" % (m.group(2), m.group(4))

    bs = getUrl(apiurl).getBS()
    data = '[Spotify] '
    if m.group(2) == 'album':
        artist = str(bs.first('album').first('artist').first('name').string)
        name = str(bs.first('album').first('name').string)
        year = str(bs.first('album').first('released').string)
        data += '%s - %s (%s)' % (artist, name, year)
    if m.group(2) == 'artist':
        data += str(bs.first('artist').first('name').string)
    if m.group(2) == 'track':
        artist = str(bs.first('track').first('artist').first('name').string)
        #album = str(bs.first('track').first('album').first('name').string)
        title = str(bs.first('track').first('name').string)
        data += '%s - %s' % (artist, title)

    return bot.say(channel, data)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

""" Pyfibot START Knowledge Parser
@author Ville 'tuhoojabotti' Lahdenvuo <tuhoojabotti@gmail.com> (http://www.tuhoojabotti.com/)
@copyright Copyright (c) 2011 Ville Lahdenvuo
@licence BSD

$Id: module_ask.py 340 2012-11-14 22:16:45Z riku.lindblad@gmail.com $
$HeadURL: https://pyfibot.googlecode.com/svn/trunk/pyfibot/modules/available/module_ask.py $
"""

import urllib
import re
import htmlentitydefs
import sys
import os
import logging
import yaml

from BeautifulSoup import BeautifulStoneSoup, Comment

# Initialize logger
log = logging.getLogger('ask')

"""Config module_ask.conf:
sentences: 1      - How many sentences of the output to print if it's longer than max length
maxlength: 150    - How many chars is the max length of output
Note: A shortlink will be applied after the maxlength e.g. See http://href.fi/xxx for more.
"""


def init(botconfig):
    global askconfig
    # Read configuration
    configfile = os.path.join(sys.path[0], 'modules', 'module_ask.conf')
    askconfig = yaml.load(file(configfile))


def command_ask(bot, user, channel, args):
    """Ask a question from the START (http://start.csail.mit.edu/) Usage: .ask <question>"""
    # SPAM!
    return bot.say(channel, getSTARTReply(args))


def getSTARTReply(q):
    if len(q) < 3 or not q:
        return "Your argument is invalid."
    # Some variables
    sentences = askconfig.get('sentences', 1)
    absmaxlen = askconfig.get('maxlength', 120)
    url = "http://start.csail.mit.edu/startfarm.cgi?QUERY=%s" % urllib.quote_plus(q)
    # For parsing
    answers = []
    data = False  # Do we have information or not
    media = False  # Do we have media such as js, img in the results
    fails = re.compile("(KNOW-DONT-KNOW|DONT-KNOW|UNKNOWN-WORD|MISSPELLED-WORD|CANT-PARSE|FORBIDDEN-ASSERTION|LEXICON)")
    medias = re.compile("doctype|click|map|below", re.IGNORECASE)
    # Retrieve data from the internet service
    bs = getUrl(url).getBS()
    if not bs:
        return "Failed to contact START. Try again later."
    # Find useful tags from the HTML mess. (Those spans with no child spans with the quality T.)
    data_tags = [tag for tag in bs(name='span', attrs={'type': 'reply', 'quality': 'T'}) if len(tag(name="span", attrs={'type': 'reply', 'quality': 'T'})) == 0]

    if len(data_tags) == 0:
        # Find tags about the users fail
        fail_tags = [tag for tag in bs(name="span", attrs={'type': 'reply', 'quality': fails}) if len(tag(name="span", attrs={'type': 'reply', 'quality': fails})) == 0]

        if len(fail_tags) == 0:
            log.debug("Failed to parse data from:")
            log.debug(bs)
            log.debug("data: %s" % data_tags)
            log.debug("fails: %s" % fail_tags)
            return "Failed to parse data. :/"
        else:  # Let's return the fail tag then.
            s = "".join([tag for tag in fail_tags[0](text=True) if type(tag) != Comment and re.search("Accept|Abort", tag) == None])
            s = re.sub("<.*?>", "", s)                          # Remove possibly remaining HTML tags (like BASE) that aren't parsed by bs
            s = re.sub("\n|\r|\t|&nbsp;", " ", s).strip(' \t')  # One-line it.
            s = re.sub("[ ]{2,}", " ", s)                       # Compress multiple spaces into one
            s = unescape(s)                                     # Clean up hex and html escaped chars
            if len(s) > absmaxlen:
                s = s[:absmaxlen].split(' ')
                s.pop()
                s = " ".join(s) + "..."
            return unicode("Fail: " + s).encode('utf-8')

    else:

        for answer in data_tags:

            # Cleanups on html depth
            [sup.replaceWith(("^%s" % sup.string) if sup.string != None else " ") for sup in answer.findAll('sup')]   # Handle <SUP> tags
            [br.replaceWith(" ") for br in answer.findAll('br')]                                                      # Handle <BR> tags
            [td.extract() for td in answer.findAll('td') if len("".join(td.findAll(text=True))) < 10]                   # Handle <TABLE> data
            [cm.extract() for cm in answer.findAll(text=lambda text:isinstance(text, Comment))]                       # Handle <!-- Comments -->

            # Find media by looking for tags like img and script and words like doctype, map, click (It sometimes embeds a whole HTML-document to the results. :S)
            if len(answer.findAll({"img": True, "script": True})) > 0 or medias.search("".join(answer(text=True))) != None:
                media = True
            # Cleanups on string depth
            s = "".join(answer(text=True))
            s = re.sub("<.*?>", "", s)                          # Remove possibly remaining HMTL tags (like BASE) that aren't parsed by bs
            s = re.sub("\n|\r|\t|&nbsp;", " ", s).strip(' \t')  # One-line it.
            s = re.sub("[ ]{2,}", " ", s)                       # Compress multiple spaces into one
            s = unescape(s)                                     # Clean up hex and html escaped chars

            answers.append(s)

        # Try to find suitable data for IRC
        try:
            answer = min((ans for ans in answers if len(ans) > 10 and not medias.search(ans)), key=len)
        except:
            if media == False:
                return "Sorry, I don't know"
            else:
                return "Take a look at %s :P" % shorturl(url).encode("utf-8")

        # Crop long answer...
        if len(answer) > absmaxlen:
            # It's longer than absolute max chars, try splitting first n sentences.
            answer = ". ".join(answer.split(". ")[:sentences]) + "."

            # It's still too long, so we'll split by word. :/
            if len(answer) > absmaxlen:
                answer = answer[:absmaxlen].split(" ")
                answer.pop()
                answer = " ".join(answer)

            answer = "%s &ndash; See %s for more." % (answer, shorturl(url))

        # It's not too long, but additional media is available, so let's give a link. :)
        elif media == True:
            answer = "%s &ndash; See %s for media." % (answer, shorturl(url))
        return unicode(unescape(answer)).encode('utf-8')


def unescape(text):
    """Unescape ugly wtf-8-hex-escaped chars"""
    def fixup(m):
        text = m.group(0)
        if text[:2] == "&#":
            # character reference
            try:
                if text[:3] == "&#x":
                    return unichr(int(text[3:-1], 16))
                else:
                    return unichr(int(text[2:-1]))
            except ValueError:
                pass
        else:
            # named entity
            try:
                text = unichr(htmlentitydefs.name2codepoint[text[1:-1]])
            except KeyError:
                pass
        return text  # leave as is
    return re.sub("&#?\w+;", fixup, text)


def shorturl(url):
    try:
        return urllib.urlopen("http://href.fi/api.php?%s" % urllib.urlencode({'create': url})).read()
    except:  # If something fails
        return url



"""
$Id: module_bmi.py 331 2012-11-13 10:18:18Z riku.lindblad@gmail.com $
$HeadURL: https://pyfibot.googlecode.com/svn/trunk/pyfibot/modules/module_bmi.py $
"""


def calc_bmi(height, weight):
    return  (float(weight) / height ** 2) * 10000


def print_bmi(bmi):
    if bmi < 16.5:
        weight_category = "severely underweight (less than 16.5)"
    elif bmi < 18.5:
        weight_category = "underweight (from 16.5 to 18.4)"
    elif bmi < 25:
        weight_category = "normal (from 18.5 to 24.9)"
    elif bmi < 30.1:
        weight_category = "overweight (from 25 to 30)"
    elif bmi < 35:
        weight_category = "obese class I (from 30.1 to 34.9)"
    elif bmi <= 40:
        weight_category = "obese class II (from 35 to 40)"
    else:
        weight_category = "obese class III (over 40)"

    return("your bmi is %.2f which is %s" % (bmi, weight_category))


def command_bmi(bot, user, channel, args):
    """Calculates your body mass index. Usage: .bmi height(cm)/weight(kg)"""
    data = args.split("/")
    if len(data) != 2:
        return bot.say(channel, "error with arguments")
    else:
        return bot.say(channel, print_bmi(calc_bmi(int(data[0]), int(data[1]))))



"""
$Id: module_autorejoin.py 331 2012-11-13 10:18:18Z riku.lindblad@gmail.com $
$HeadURL: https://pyfibot.googlecode.com/svn/trunk/pyfibot/modules/module_autorejoin.py $
"""

# rejoin after 1 minute
delay = 60

from twisted.internet import reactor


def handle_kickedFrom(bot, channel, kicker, message):
    """Rejoin channel after 60 seconds"""
    bot.log("Kicked by %s from %s. Reason: %s" % (kicker, channel, message))
    bot.log("Rejoining in %d seconds" % delay)
    bot.network.channels.remove(channel)
    reactor.callLater(delay, bot.join, channel)



"""
$Id: module_pgsqlwanha.py 331 2012-11-13 10:18:18Z riku.lindblad@gmail.com $
$HeadURL: https://pyfibot.googlecode.com/svn/trunk/pyfibot/modules/available/module_pgsqlwanha.py $
"""

import time
import datetime
try:
    from pyPgSQL import PgSQL
except:
    pass


# -- For postgresql
#
# CREATE TABLE pyfibot.urls
# (
#       url text NOT NULL,
#         channel character varying(255) NOT NULL,
#         userid character varying(255) NOT NULL,
#         "timestamp" timestamp without time zone NOT NULL,
#         id serial NOT NULL,
#         CONSTRAINT id PRIMARY KEY (id)
#       )
# WITHOUT OIDS;
# ALTER TABLE pyfibot.urls OWNER TO <user>;

def init(bot):
    global config
    config = bot.config.get("module_pgsqlwanha", None)


def handle_url(bot, user, channel, url, msg):
    return
    if not config:
        return
    cx = PgSQL.connect(database=config["database"], host=config["host"], user=config["user"], password=config["password"])
    cur = cx.cursor()
    # find the oldest instance of given url on this channel
    cur.execute("SELECT * FROM pyfibot.urls WHERE channel=%s AND url=%s ORDER BY timestamp;", (channel, url))
    res = cur.fetchone()
    if res:
        url, channel, userhost, timestamp, urlid = res
        pastetime = datetime.datetime.fromtimestamp(res[3].ticks())
        now = datetime.datetime.now()
        age = now - pastetime
        agestr = ""
        if age.days > 0:
            agestr += "%d days " % age.days
        secs = age.seconds
        hours, minutes, seconds = secs // 3600, secs // 60 % 60, secs % 60
        if hours > 0:
            agestr += "%d h " % hours
        if minutes > 0:
            agestr += "%d m " % minutes
        if seconds > 0:
            agestr += "%d s" % seconds
        # don't alert for the same person
        if getNick(user) != getNick(userhost):
            if channel != "#wow":
                bot.say(channel, "%s: wanha. (by %s %s ago)" % (getNick(user), getNick(userhost), agestr))
    cur = cx.cursor()
    # always store URLs, this structure can handle it, sqlite can't
    cur.execute("INSERT INTO pyfibot.urls (userid, channel, url, timestamp) VALUES(%s, %s, %s, NOW());", (user, channel, url))
    cx.commit()
    cur.close()
    cx.close()




"""A basic library for getting data out of epguides.com"""

import re, urllib, string, time, datetime
import ConfigParser, os.path, sys, md5

class Serie:
    """One serie"""
    
    def __init__(self, webname, name):
        self.name = name
        self.webname = webname
        self.checksum = None
        self.epdata = []

    def __str__(self):
        return self.__repr__()

    def __repr__(self):
        return "<%s (%d eps)>" % (self.name, len(self.epdata))

def getConfig(iniFileName="epwatcher.ini"):
    """Reads a config file of <name in epguides>=<display name> -pairs and returns it as a dict"""
    
    # read config with config parser
    cp = ConfigParser.RawConfigParser()
    # make keys case sensitive
    cp.optionxform = lambda x: str(x)

    # try to find the ini file
    cp.read([iniFileName, os.path.join(sys.path[0], iniFileName), os.path.join(sys.path[0], iniFileName)])

    series = {}

    # series to watch
    if cp.has_section('series'):
        for k,v in cp.items('series'):
            series[k] = v

    return series

def printSerieData(series, outputstring, daysbefore=1, daysafter=3):
    """Print serie data in a neat form to stdout

    @param seriedata Old serie data for initialization
    
    """
    now = time.gmtime()

    # datetime format for easy comparisons
    nowdate = datetime.date(now[0], now[1], now[2])
    yesterday = nowdate - datetime.timedelta(days=1)
    tomorrow = nowdate + datetime.timedelta(days=1)

    comingupseries = []

    # analyze
    for serie in series:
        for ep in serie.epdata:
            airdate = ep['airdate2']
            # create a timedelta object of the two dates
            # and compare to the given ranges
            td = airdate - nowdate
            if td.days < daysafter and td.days >= -daysbefore:
                comingupseries.append( (ep['airdate2'], serie.name, ep['season'], ep['episode'], ep['epname']) )

    # sort by first item (date)
    comingupseries.sort()

    # the output string needs some preformatting to handle long serie names
    outputstring = outputstring.replace("%(", "%%(")

    maxlen = 0

    for airdate, seriename, season, episode, epname in comingupseries:
        if len(seriename) > maxlen: maxlen = len(seriename)


    outputstring = outputstring % (-maxlen)

    # go through all the series which are on during the given timeframe
    for airdate, seriename, season, episode, epname in comingupseries:
        # format the airdate to a readable form
        if airdate == nowdate:
            airdate = "TODAY"
        elif airdate == yesterday:
            airdate = "YESTERDAY"
        elif airdate == tomorrow:
            airdate = "TOMORROW"
        else:
            airdate = airdate.strftime("%d.%m.%Y")

        # print the formatted string
        print outputstring % locals()

def getSerieData(serie, proxies=None, url="http://www.epguides.com/"):
    """Get all data for given series from epguides.com

    @param series a dict of {<name on epguides>:<long name>, ...}
    """
    
    # line example
    # 1.   1- 1       535J     30 Sep 01   <a href="http://www.tvtome.com/Alias/season1.html#ep1">Truth Be Told</a>
    epRe=re.compile("(?P<no>[\d]+)\.[\s]+(?P<season>[\d]+)-[ ]?(?P<episode>[\d]+)[\s]+(?P<prodno>[\w-]+)?[\s]+(?P<date>[\d]{1,2} [\w]{3} [\d]{2})[\s]+<[^>]+>(?P<name>.*)</a>")

    # set proxy here if needed
    f=urllib.urlopen("%s%s/" % (url, serie.webname), proxies=proxies)
    filedata=f.read()
    f.close()

    md5sum = md5.new(filedata).hexdigest()

    # no changes, don't update
    if serie.checksum == md5sum:
        return serie

    serie.checksum = md5sum
    
    # {<seriename> : [{epdata}, {epdata}, {epdata}], ...}
    
    tmp=filedata.split("\n")
    # if the file didn't split, try again
    # with a different separator
    if int(len(tmp)) == 1:
        tmp=filedata.split("\r")
    filedata=tmp

    # remove leading and trailing spaces
    filedata=map(string.strip, filedata)

    # remove old data
    serie.epdata = []

    for line in filedata:
        ###
        # find episodes
        ###
        m = epRe.match(line)
        if m:
            
            # convert datestring to gmtime format
            t = time.strptime(m.group('date'), '%d %b %y')
            
            # put episode data into a nifty dict
            data = {'epno'    : m.group('no'),
                    'season'  : int(m.group('season')),
                    'episode' : int(m.group('episode')),
                    'prodno'  : m.group('prodno'),
                    'airdate' : m.group('date'),
                    'airdate2': datetime.date(t[0], t[1], t[2]),
                    'epname'  : m.group('name')}
        
            serie.epdata.append(data)

        
    return serie



# -*- coding: utf-8 -*-

"""
$Id: module_weather.py 331 2012-11-13 10:18:18Z riku.lindblad@gmail.com $
$HeadURL: https://pyfibot.googlecode.com/svn/trunk/pyfibot/modules/module_weather.py $
"""

has_pywapi = False
try:
    import pywapi
    has_pywapi = True
except:
    print('Error loading library pywapi. Probably you havent installed it yet.')


def feels_like(temperature, air_speed):
    """tempeture as °C, air_speed as kmph. The formula from http://www.bbc.co.uk/weather/features/understanding/feelslike.shtml"""
    air_speed = air_speed ** 0.16
    return (13.12 + 0.6215 * temperature - 11.37 * air_speed + 0.3965 * temperature * air_speed)


def parse_google_wind_condition(wind_condition):
    """ Parses the data from string """
    wind_condition = wind_condition.split(' ')
    kmph = int(wind_condition[3]) * 1.609344
    mps = kmph / 3.6
    return wind_condition[1], kmph, mps


def command_weather(bot, user, channel, args):
    """ This module tells a weather for location."""
    if not has_pywapi:
        return
    result_dict = pywapi.get_weather_from_google(args)
    if not all(result_dict.values()):
        bot.say(channel, 'unknown location')
        return
    compass_point, kmph, mps = parse_google_wind_condition(result_dict['current_conditions']['wind_condition'])
    compass_points = {'N': 'north', 'S': 'south', 'W': 'west', 'E': 'east', 'NW': 'northwest', 'NE': 'northeast', 'SW': 'southwest', 'SE': 'southeast'}
    compass_point = compass_points[compass_point]
    city = result_dict['forecast_information']['city']
    humidity = result_dict['current_conditions']['humidity']
    condition = result_dict['current_conditions']['condition']
    temperature = int(result_dict['current_conditions']['temp_c'])
    answer = u'%s: %s, %d °C feels like %d °C, Wind: %.1f m/s from %s, %s' % \
        (city, condition, temperature, feels_like(temperature, kmph), mps, compass_point, humidity)
    answer = answer.encode("utf-8")
    return bot.say(channel, answer)
